use Test;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::CodeGenerator::API::CreateCode qw ( CreateCode );

our @Files;
our $TEST_ONE;

BEGIN {
    local $SIG{__DIE__}  = \&DE_EPAGES::Core::API::Error::DieErrorHandler;
    local $SIG{__WARN__} = \&DE_EPAGES::Core::API::Error::DieErrorHandler;

    @Files = glob '*.pdm';
    plan tests => (defined $TEST_ONE ? 1 : scalar @Files);
};

sub Main {
    Initialize();
    if ( defined $TEST_ONE ) {
        ok ( Test_Package($TEST_ONE ) );
    } else {
        foreach my $File ( @Files ) {
            die "invalid file name $File" unless $File =~ /^(.+)\.pdm$/;
            my $TestCase = $1;
            ok ( Test_Package( $TestCase ) );
        }
    }
}

local $| = 1;
eval {
    Main(@ARGV);
};

if (ExistsError()) {
    print GetError()->longMessage(),"\n";
    exit 1;
} else {
    print ".\n";
}

sub ErrorDefinition { return {
    NO_EQUAL_LINE => "Output is different at line #AtLine"
}; }

sub Test_Package {
    my ($i) = @_;
    eval {
        CreateCode({
            'BUILD_FILE'        => '../Templates/epages/Build.xml',
            'DEBUG'             => 0,
            'SOURCE_FILE'       => $i.'.pdm',
            'TARGET_DIR'        => '.',
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub Initialize {
    $bin::ISSRV::AppServerIP = 12.12.12.12 if not defined $bin::ISSRV::AppServerIP;
    $bin::ISSRV::AppServerPort = 1234 if not defined $bin::ISSRV::AppServerPort;
    local $ENV{SCRIPT_NAME} = 'CodeGenerator.t';

}

sub Cleanup {
}

1;

