use strict;
use Test::Simple tests => 15;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    ok( $Shop->alias eq 'TestShop', "Shop loaded" );

    my $RootCategory = $Shop->get('Categories');
    $RootCategory->child('TEST')->delete if $RootCategory->existsChild('TEST');

    my $TestCategory = $Shop->get('Categories')->insertChild('Category', { Alias => 'TEST' });
    ok( $RootCategory->existsChild('TEST'), "TEST category exists" );
    ok( !$TestCategory->get('IsVisible'), "new category is invisible by default" );

    # create 2 levels of sub categories
    my $Sub1 = $TestCategory->insertChild('Category', { Alias => 'Sub1', Position => 30, IsVisible => 1 });
    my $Sub2 = $TestCategory->insertChild('Category', { Alias => 'Sub2', Position => 20, IsVisible => 1 });
    my $Sub3 = $TestCategory->insertChild('Category', { Alias => 'Sub3', Position => 10, IsVisible => 0 });
    my $Sub11 = $Sub1->insertChild('Category', { Alias => 'Sub11', Position => 30, IsVisible => 1 });
    my $Sub12 = $Sub1->insertChild('Category', { Alias => 'Sub12', Position => 20, IsVisible => 1 });
    my $Sub13 = $Sub1->insertChild('Category', { Alias => 'Sub13', Position => 10, IsVisible => 0 });

    my $aSubCategories = $TestCategory->get('SubCategories');
    ok( 3 == scalar @$aSubCategories, "number of direct sub categories" );
    ok( $aSubCategories->[0]->alias eq 'Sub3', "sort order of sub categories" );
    ok( $aSubCategories->[1]->alias eq 'Sub2', "sort order of sub categories" );

    my $aRecursiveSubCategories = $TestCategory->get('RecursiveSubCategories');
    ok( 6 == scalar @$aRecursiveSubCategories, "number of recursive sub categories" );
    ok( $aRecursiveSubCategories->[0]->alias eq 'Sub3', "sort order of recursive sub categories" );
    ok( $aRecursiveSubCategories->[5]->alias eq 'Sub11', "sort order of recursive sub categories" );

    my $aVisibleSubCategories = $TestCategory->get('VisibleSubCategories');
    ok( 2 == scalar @$aVisibleSubCategories, "number of visible sub categories" );
    ok( $aVisibleSubCategories->[0]->alias eq 'Sub2', "sort order of visible sub categories" );
    ok( $aVisibleSubCategories->[1]->alias eq 'Sub1', "sort order of visible sub categories" );

    $TestCategory->delete;
    ok( !$Shop->get('Categories')->existsChild('TEST'), "category deleted" );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
