use Test::Simple tests => 20;

use strict;
use utf8;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::IDN qw(
    EncodeDomain
    DecodeDomain
    EncodeDomainInUrl
    DecodeDomainInUrl
    EncodeDomainInEMailAddress
    DecodeDomainInEMailAddress
    AreDomainsEqual
    IsValidIDN
);

sub Main {
    RunTests( "EncodeDomain", [
        [ 'müller.de' ], [ 'xn--mller-kva.de' ],
        [ 'MÜLLER.de' ], [ 'xn--mller-kva.de' ],
        [ 'MuELlEr.de' ], [ 'mueller.de' ],
        [ 'äöüß.de' ], [ 'xn--ss-uia6e4a.de' ],
        [ 'Sørensen.dk' ], [ 'xn--srensen-q1a.dk' ],
        [ 'BÜcher.müLLEr.de' ], [ 'xn--bcher-kva.xn--mller-kva.de' ],
        [ 'waßER.de' ], [ 'wasser.de' ],
              ]);
    RunTests( "DecodeDomain", [
        [ 'xn--mller-kva.de' ], [ 'müller.de' ],
              ]);
    RunTests( "EncodeDomainInUrl", [
        [ 'https://shop.müller.de/shop1.admin' ], [ 'https://shop.xn--mller-kva.de/shop1.admin' ],
        [ 'https://BÜcher:müLLEr@shop.müller.de/shop1.admin' ], [ 'https://BÜcher:müLLEr@shop.xn--mller-kva.de/shop1.admin' ],
              ]);
    RunTests( "DecodeDomainInUrl", [
        [ 'https://shop.xn--mller-kva.de/shop1.admin' ], [ 'https://shop.müller.de/shop1.admin' ],
        [ 'https://BÜcher:müLLEr@shop.xn--mller-kva.de/shop1.admin' ], [ 'https://BÜcher:müLLEr@shop.müller.de/shop1.admin' ],
              ]);
    RunTests( "EncodeDomainInEMailAddress", [
        [ 'webmaster@müller.de' ], [ 'webmaster@xn--mller-kva.de' ],
        [ 'web(@).müller@müller.de' ], [ 'web(@).müller@xn--mller-kva.de' ],
              ]);
    RunTests( "DecodeDomainInEMailAddress", [
        [ 'webmaster@xn--mller-kva.de' ], [ 'webmaster@müller.de' ],
        [ 'web(@).müller@xn--mller-kva.de' ], [ 'web(@).müller@müller.de' ],
              ]);
    RunTests( "AreDomainsEqual", [
        [ 'müller.de', 'xn--mller-kva.de' ], [ 1 ],
              ]);
    RunTests( "IsValidIDN", [
        [ 'www.vanitydomainshop.de' ], [ 1 ],
        [ 'MÜLLER.de' ], [ 1 ],
        [ 'BÜcher.müLLEr.de' ], [ 1 ],
              ]);
}

RunScript( Sub => \&Main, Message => 'long' );

sub RunTests {
    ##no critic(ProhibitNoStrict)
    no strict;
    ##use critic
    my( $Run, $Tests ) = @_;

    for (my $i = 0; $i < scalar(@$Tests); $i += 2) {
        my $Parameters = $Tests->[$i];
        my $ExpectedResult = $Tests->[$i+1];
        my @ActualResult;
        eval { @ActualResult = &$Run(@$Parameters); };

        my $OK = @$ExpectedResult ~~ @ActualResult;
        ok($OK, "$Run(". join(",",@$Parameters). ") = ".
           join(",",@$ExpectedResult). " -> ". join(",",@ActualResult));
    }
}
