use Test::More tests => 17;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::InlineJavaServer qw( ExistsRunningInlineJavaServer StartInlineJavaServer StopInlineJavaServer );
use DE_EPAGES::Core::API::PdfConverter qw( IsValidXhtml ConvertXhtml2Pdf CloseHtmlTags );

RunScript( Sub => sub {

    StopInlineJavaServer();

    my $HtmlCode = '<html><head></head><body><p>blubb</p></body></html>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( IsValidXhtml($HtmlCode), "valid xhtml" );

    $HtmlCode = '<html><head></head><body><p>blubb</body></html>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode), "invalid xhtml" );

    $HtmlCode = '<html><head></head><body att=sddf><p>blubb</p></body></html>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode), "invalid xhtml" );

    $HtmlCode = '<html><head></head><body att=sddf"><p>blubb</p></body></html>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode), "invalid xhtml" );

    $HtmlCode = '<html><head></head><body att="sddf><p>blubb</p></body></html>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode), "invalid xhtml" );

    $HtmlCode = '<html><head></head><body att="sddf"<p>blubb</p></body></html>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode), "invalid xhtml" );

    $HtmlCode = '<html><head></head><body att="sddf"><p>blubb</p></body>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode), "invalid xhtml" );

    $HtmlCode = '<head></head><body att="sddf"><p>blubb</p></body></html>';
    print "\n> Test xhtml validator: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode), "invalid xhtml" );

    $HtmlCode = '<p>blubb</p>';
    print "\n> Test xhtml validator with flag isSnippet: ", $HtmlCode, "\n";
    ok( IsValidXhtml($HtmlCode, {'isSnippet' => 1}), "valid xhtml" );

    $HtmlCode = '<p>blubb';
    print "\n> Test xhtml validator with flag isSnippet: ", $HtmlCode, "\n";
    ok( !IsValidXhtml($HtmlCode, {'isSnippet' => 1}), "invalid xhtml" );

    $HtmlCode = '<html><head><body att=sdsd><p>blubb';
    my $Exp = '<html><head></head><body att="sdsd"><p>blubb</p></body></html>';
    print "\n> Test Function CloseHtmlTags: ", $HtmlCode, "\n";
    my $Output = CloseHtmlTags($HtmlCode);
    chomp($Output);
    ok( $Output eq $Exp, $Output );

    $HtmlCode = '<html><head></head><body att="sddf"><p>blubb</p></body></html>';
    print "\n> Test pdf converter on valid html: ", $HtmlCode, "\n";
    $Output = ConvertXhtml2Pdf($HtmlCode);
    ok( $Output =~ /^%PDF/, "got pdf document" );

    $HtmlCode = '<html><head></head><body att="sddf><p>blubb</p></body></html>';
    print "\n> Test pdf converter on invalid html: ", $HtmlCode, "\n";
    $Output = ConvertXhtml2Pdf($HtmlCode);
    ok( (not defined $Output), "got undef" );

    print "\n>> Test Inline::Java server..\n";
    print "\n> Is server running?\n";
    ok( ExistsRunningInlineJavaServer(), "server is running" );

    print "\n> Shutdown server if it is running!\n";
    StopInlineJavaServer();
    sleep(2);
    ok( !ExistsRunningInlineJavaServer(), "server is not running" );

    print "\n> Startup server if it is not running.\n";
    StartInlineJavaServer();
    sleep(2);
    ok( ExistsRunningInlineJavaServer(), "server is running" );

    print "\n> Shutdown server if it is running and try to create an pdf document.\n";
    print "> ..JVM should start automatically!\n";
    $HtmlCode = '<html><head></head><body att="sddf"><p>blubb</p></body></html>';
    StopInlineJavaServer();
    sleep(2);
    $Output = ConvertXhtml2Pdf($HtmlCode);
    ok( $Output =~ /^%PDF/, "got pdf document" );

    StopInlineJavaServer();

});
