﻿#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Url qw(
    UrlEncodeValue
    BuildParameterString
    StringIsURL
);

plan tests => 15;

# UrlEncodeValue
# the following characters will NOT be encoded:
#     "-" / "." / "_" / "~"
#     gen-delims  = ":" / "/" / "?" / "#" / "[" / "]" / "@"
# while these reserved characters have to be encoded:
#     sub-delims  = "!" / "$" / "&" / "'" / "(" / ")"
#                 / "*" / "+" / "," / ";" / "="
# UrlEncodeValue re-decodes the slash / ..
eval {
    UrlEncodeValue();
};
if(ExistsError()) {
    my $Error = GetError();
    cmp_ok($Error->code, 'eq', 'PARAMETER_MISSING', 'UrlEncodeValue undef input');
}
cmp_ok(UrlEncodeValue(''), 'eq', '', 'UrlEncodeValue empty input');
cmp_ok(UrlEncodeValue('-._~:/?#[]@!$&\'()*+,;="Test//_Str i n G%12 '), 'eq',
    '-._~%3A/%3F%23%5B%5D%40!%24%26\'()*%2B%2C%3B%3D%22Test//_Str%20i%20n%20G%2512%20',
    'UrlEncodeValue teststring');
cmp_ok(UrlEncodeValue('-._~:/?#[]@!$&\'()*+,;="Test//_Str i n G%12 Αθηνά', 1), 'eq',
    '-._~%3A/%3F%23%5B%5D%40!%24%26\'()*%2B%2C%3B%3D%22Test//_Str%20i%20n%20G%2512%20%CE%91%CE%B8%CE%B7%CE%BD%CE%AC',
    'UrlEncodeValue teststring noutf8');
cmp_ok(UrlEncodeValue('?pageid=10&lang=en&title=Section%20Title</h1><script>alert(\'XSS%20attack\')</script>'),'eq',
    '%3Fpageid%3D10%26lang%3Den%26title%3DSection%2520Title%3C/h1%3E%3Cscript%3Ealert(\'XSS%2520attack\')%3C/script%3E',
    'UrlEncodeValue script tag');

# BuildParameterString
cmp_ok(BuildParameterString({'ReturnUrl' => 'http://www.shop.de/epages/?ChangeAction=Change&ViewAction=View'}), 'eq',
                                           'ReturnUrl=http%3A//www.shop.de/epages/%3FChangeAction%3DChange%26ViewAction%3DView',
                                           'BuildParameterString hash returnurl');
cmp_ok(BuildParameterString({'ChangeAction' => ['BlockMonitor', 'Save'], 'ViewAction' => 'View'}), 'eq',
                                           'ChangeAction=BlockMonitor&ChangeAction=Save&ViewAction=View',
                                           'BuildParameterString hash actions');

# StringIsURL
is(StringIsURL('my/dir/file.jpg'), 0, 'StringIsURL relative filepath');
is(StringIsURL('c:\\dir\\file.jpg'), 0, 'StringIsURL abs filepath');
is(StringIsURL('http_www_url.jpg'), 0, 'StringIsURL filename');
is(StringIsURL('www.myurl.com'), 0, 'StringIsURL www domain');
is(StringIsURL('myurl.com/my/dir'), 0, 'StringIsURL domain with subdir');
is(StringIsURL('http://myurl.com/my/dir'), 1, 'StringIsURL http domain with subdir');
is(StringIsURL('http://127.0.0.1'), 1, 'StringIsURL http localhost');
is(StringIsURL('https://www.mydomain.de'), 1, 'StringIsURL https domain');

