/*
  Copyright (c) 2006-2010, epages Software GmbH
  All Rights Reserved.

  +-------------------------------------------------------------------+
  | headlineText                                                      |
  |                                                                   |
  |         +-------------------+                                     |
  |   name  | description Edit  | binIcon | moveUpIcon | moveDownIcon |
  |         +-------------------+                                     |
  |                                                                   |
  |         +-------+                                                 |
  |         | Add   |                                                 |
  |         +-------+                                                 |
  +-------------------------------------------------------------------+
*/
dojo.provide("epages.cartridges.de_epages.crossselling.widget.CrossSellingWizard");//an dieser Stelle wird das neue Widget registriert
dojo.require("epages.widget.LocalizedWidget");

dojo.declare("epages.cartridges.de_epages.crossselling.widget.CrossSellingWizard", [epages.widget.LocalizedWidget], {
    templatePath    : dojo.moduleUrl('epages.cartridges.de_epages.crossselling.widget', "templates/CrossSellingWizard.html"),
    translationName : dojo.moduleUrl('epages.cartridges.de_epages.crossselling.widget', "templates/CrossSellingWizard"),

    /*
        public properties
    */
    headlineText      : "",
    dialogTitleText   : "", // string - title text of add item dialog
    remarkText        : "", // string - input label
    editable          : true,
    actionName        : '',
    resourceObjectID  : '',
    inputBaseName     : '',

    /*
        private properties
    */
    //Style settings
    _editFieldWidth   : 490, //width in px
    _nameFieldWidth   : 200, //width in px

    _currentPosition  : 10,
    _currentID        : 0,
    _eventHandles     : {},
    _actionRunning    : false,

    postMixInProperties : function(){
      this.inherited("postMixInProperties", arguments);
    },

    postCreate  : function() {
      this.inherited("postCreate", arguments);
      this.headline.innerHTML = this.headlineText;

      //move button to a nice place

      //attach click event
      dojo.connect(this.formAddButton, 'onclick', this, 'addItem');
      dojo.subscribe(this.id+'/add',this,"addNewContent");
    },

    addItem : function(){
      //summary: connect to the publish function to use the add button to display something
      dojo.publish(this.id+'/addPressed',[this.id, this.dialogTitleText]);
    },

    deleteContent : function(/* Object*/ evt){
      if(this._actionRunning){ return; }
      this._actionRunning = true;
      // summary: remove a content node from the content table after the bin button is pressed
      var target = dojo.query("[name='crossSellingContent"+evt.currentTarget.getAttribute("contentID")+"']",this.contentTable)[0];

      // make removing smooth
      var wipeOut = dojo.fx.wipeOut({node: target,duration: 500});
      var handle = dojo.connect(wipeOut, "onEnd", this, function(){
          this.contentTable.removeChild(target);
          //remove connected events
          dojo.forEach(this._eventHandles["'"+evt.currentTarget.getAttribute("contentID")+"'"],
                      function(e){
                        dojo.disconnect(e);
                      }
          );
          delete this._eventHandles["'"+evt.currentTarget.getAttribute("contentID")+"'"];
          this._actionRunning = false;
      });
      wipeOut.play();
    },

    _switchNodes : function(/* Object*/ first,/* Object*/ second){
      if(this._actionRunning){ return; }
      this._actionRunning = true;
      // summary: exchange two DOM nodes in the content table
      var wipeOut = dojo.fx.wipeOut({node: first,duration: 500});
      var wipeIn = dojo.fx.wipeIn({node: first,duration: 500});

      //make the change smooth
      var handle = dojo.connect(wipeOut, "onEnd", this, function(){
        this.contentTable.removeChild(second);
        this.contentTable.insertBefore(second,first);
        wipeIn.play();
        dojo.disconnect(handle);
        this._actionRunning = false;
      });
      wipeOut.play();

      var positionNodeFirst = dojo.query("input[name='"+this.resourceObjectID+":"+this.inputBaseName+"Position']",first);
      var positionNodeSecond = dojo.query("input[name='"+this.resourceObjectID+":"+this.inputBaseName+"Position']",second);

      // exchange the value of the hidden field which represents the index of the nodes
      var buffer = positionNodeFirst[0].getAttribute("value");
      positionNodeFirst[0].setAttribute("value",positionNodeSecond[0].getAttribute("value"));
      positionNodeSecond[0].setAttribute("value",buffer);
    },

    moveContentUp : function(/* Object*/ evt){
      // summary: increase a content nodes index to display it in front of the previous node
      var currentNode = dojo.query("[name='crossSellingContent"+evt.currentTarget.getAttribute("contentID")+"']",this.contentTable)[0];
      var previouseNode = currentNode.previousSibling;
      //prevent to switch places with non-content nodes
      if(previouseNode && dojo.hasClass(previouseNode,"CrossSellingItem")){
        this._switchNodes(previouseNode,currentNode);
      }
    },

    moveContentDown : function(/* Object*/ evt){
      // summary: decrease a content nodes index to display it after the next node
      var currentNode = dojo.query("[name='crossSellingContent"+evt.currentTarget.getAttribute("contentID")+"']",this.contentTable)[0];
      var nextNode = currentNode.nextSibling;
      //prevent to switch places with the "add" Button
      if(nextNode && dojo.hasClass(nextNode,"CrossSellingItem")){
        this._switchNodes(currentNode,nextNode);
      }
    },
    checkIfAlreadyPresent : function(objectID){
      // summary: prevent adding one product more then once
      var contentObjects = dojo.query("input[name='ObjectID']",this.contentTable);
      for(var i=0;i<contentObjects.length;i++){
        if(objectID == contentObjects[i].value){
          return true;
        }
      }
      return false;
    },
    addNewContent : function(/* Object*/ content){
      // summary: add a new content line to table
      // content:
      // - ObjectID id of the product to offer
      // - NameOrAlias Name of the product to offer
      // - Alias alias of the product to offer
      // - desc description of the offer (optional)
      if(this.checkIfAlreadyPresent(content.ObjectID)){
        return;
      }
      var contentNode = document.createElement('div');
      contentNode.className="CrossSellingItem BorderRadius6Box Grey NormalPadding";
      var contentHTML = '';
      var additionalLanguages = '';
      var timestamp = (new Date()).getTime();
      var node = undefined;

      //Create node, push node in DOM
      contentNode.setAttribute("name","crossSellingContent"+this._currentID);
      contentHTML='<table><tr>';

      if(this.editable){
        // Is the content description going to be editable?
        contentHTML += '<td class="Label" style="width:'+this._nameFieldWidth+'px;"><br />'+content.NameOrAlias+
        '<input type="hidden" name="'+this.resourceObjectID+':'+this.inputBaseName+'NewProduct" value="'+content.Alias+'" />'+
        '</td>';
        contentHTML += '<td style="text-align:left;width:'+(this._editFieldWidth+10)+'px;">';
				contentHTML += '<span class="AdditionalInfo">' + this.remarkText + '</span><br />';
        contentHTML += '<input type="text" style="width:'+this._editFieldWidth+'px;" dojoType="epages.widget.FormElement" name="'+this.resourceObjectID+':'+this.inputBaseName+'Comment:'+epages.vars.Locale.languageId+':'+this.inputBaseName+'LanguageID" value="'+(content.desc || "")+'">';
        contentHTML += '<input type="hidden" name="'+this.resourceObjectID+':'+this.inputBaseName+'CommentTargetAlias" value="'+content.Alias+'" />';
        for(var lange in epages.vars.languages){
          if(lange != epages.vars.Locale.languageId){
            additionalLanguages += '<br />';
            additionalLanguages += '<input type="text" style="width:'+this._editFieldWidth+'px;" value="" name="'+this.resourceObjectID+':'+this.inputBaseName+'Comment:'+lange+':'+this.inputBaseName+'LanguageID" id="Comment:'+lange+'">';
            additionalLanguages += '<input type="hidden" name="'+this.resourceObjectID+':'+this.inputBaseName+'CommentTargetAlias" value="'+content.Alias+'" />';
            additionalLanguages += '<br/ ><label for="Comment:'+lange+'" class="Smaller">('+epages.vars.languages[lange]["native"]+')</label>';
          }
        }
        if(additionalLanguages!=""){
          contentHTML += '<br /><div class="AlignRight"><span class="ChangeLink Smaller" id="moreLanguageDescriptions'+timestamp+'">(<a href="#" onclick="showMore(\'moreCrossSellingDescriptions'+timestamp+'\', $(\'moreLanguageDescriptions'+timestamp+'\') ); return false;">'+this.translate("MoreLanguages")+'</a>)</span></div>';
          contentHTML += '<div id="moreCrossSellingDescriptions'+timestamp+'" style="display:none">';
          contentHTML += additionalLanguages;
          contentHTML += '</div>';
        }
        contentHTML += '</td>';
      } else {
        // Content is not editable
        contentHTML += '<td style="width:'+(this._nameFieldWidth+10+this._editFieldWidth)+'px;"><span class="Label"><b>'+content.NameOrAlias;
        contentHTML += '</b></span><br />';
        contentHTML += '<input type="hidden" name="'+this.resourceObjectID+':'+this.inputBaseName+'NewProduct" value="'+content.Alias+'" />';

        contentHTML += '<span>'+(content.desc || "")+'</span>';
        contentHTML += '</td>';
      }

      contentHTML += '<td><input type="hidden" name="'+this.resourceObjectID+':'+this.inputBaseName+'Position" value="'+this._currentPosition+'"></td>';

      contentHTML += '<td><br /><i class="Sprite ico_m_delete ClickIcon" contentID="'+this._currentID+'" name="deleteCrossSelling'+this._currentID+'" title="'+this.translate("Delete")+'"></i></td>';

      contentHTML += '<td><br /><i class="Sprite ico_m_move_up ClickIcon" contentID="'+this._currentID+'" name="upCrossSelling'+this._currentID+'" title="'+this.translate("MoveUp")+'"></i></td>';

      contentHTML += '<td><br /><i class="Sprite ico_m_move_down ClickIcon" contentID="'+this._currentID+'" name="downCrossSelling'+this._currentID+'" title="'+this.translate("MoveDown")+'"></i></td>';

      contentHTML +='</tr></table>';

      contentNode.innerHTML = contentHTML;

      //add to DOM
      this.contentTable.insertBefore(contentNode,this.formAddButtonDiv);
      var handles = new Array();
      handles.push(dojo.connect(dojo.query("[name='deleteCrossSelling"+this._currentID+"']",this.contentTable)[0],
                  'onclick', this, 'deleteContent'));
      handles.push(dojo.connect(dojo.query("[name='upCrossSelling"+this._currentID+"']",this.contentTable)[0],
                  'onclick', this, 'moveContentUp'));
      handles.push(dojo.connect(dojo.query("[name='downCrossSelling"+this._currentID+"']",this.contentTable)[0],
                  'onclick', this, 'moveContentDown'));

      this._eventHandles["'"+this._currentID+"'"] = handles;

      this._currentPosition +=10;
      this._currentID++;
    }
  }
);