use Test::More;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::DHL::API::Tools qw( GetDHLProductMaxDimensionsFromDB );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );

#--------------------------------------------------------------------------------------
sub testGetMaxDimensions {
    my $DHLProductID = shift;
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            is(
                fetchDimensionString('PAECK.DEU'),
                'PAECK.DEU // 60 // 30 // 15',
                'PAECK.DEU'
            );

            is(
                fetchDimensionString('PAK10.DEU'),
                'PAK10.DEU // 120 // 60 // 60',
                'PAK10.DEU'
            );

            is(
                fetchDimensionString('JS_PRODUCT_NOT_IN_DB.DEU'),
                '--- // 0 // 0 // 0',
                'JS_PRODUCT_NOT_IN_DB.DEU'
            );

            is(
                fetchDimensionString('PAK05.AUT'),
                "--- // 0 // 0 // 0",
                "PAK05.AUT"
            );

            is(
                fetchDimensionString('PAK05.EU'),
                "PAK05.EU // 120 // 60 // 60",
                "PAK05.EU"
            );
        }
    );
}

sub fetchDimensionString {
    my ($DHLProductID)= @_; 
    my $hDHLProductMaxDimensions = GetDHLProductMaxDimensionsFromDB($DHLProductID);

    my $ResultProductID = '';
    if (defined($hDHLProductMaxDimensions->{'DHLProductID'})) {
        $ResultProductID = $hDHLProductMaxDimensions->{'DHLProductID'};
    }
    else {
        $ResultProductID = '---';
    }

    my $ResultStr = $ResultProductID." // ".$hDHLProductMaxDimensions->{'MaxLengthCm'}." // ".$hDHLProductMaxDimensions->{'MaxWidthCm'}." // ".$hDHLProductMaxDimensions->{'MaxHeightCm'};
    return $ResultStr;
}

#--------------------------------------------------------------------------------------
RunScript(
    'Sub' => sub {
        testGetMaxDimensions;
        done_testing;
        return;
    }

);
