use Test::Simple tests => 3;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw (   ConnectToSite DisconnectSite
                                                EncryptDBPassword);
use DE_EPAGES::Database::API::Config qw( DatabaseConfig DatabaseConnectionData );

sub Main {
    Initialize();
    ok ( Test_Package1() );
    ok ( Test_Package2() );
    ok ( Test_DBPasswordEncryption() );
}

RunScript( Sub => \&Main, Debug => 1 );

sub Test_Package1 {
    eval {
        ConnectToSite();
        DisconnectSite();

    };
    if (ExistsError()) {
        my $Error = GetError();
        return 0;
    }
    return 1;
}

sub Test_Package2 {
    eval {
        # ignore warning for DBI connect failed
        local $SIG{__WARN__} = sub {};
        ConnectToSite('thomas');
        DisconnectSite();
    };
    if (ExistsError()) {
        my $Error = GetError();
        return 0 if $Error->code ne 'DBIConnect';
        return 1;
    }
    return 0;
}

sub Test_DBPasswordEncryption {
    eval {
        my $IniConfig = DatabaseConfig('Site');

        # Connect first time and store plaintext password encrypted
        ConnectToSite();
        my $hSection = DatabaseConnectionData('Site');
        my $PlainPassword =  $hSection->{'passwd'};
        Error("Passwords are already encrypted!") unless $PlainPassword;
        my $EncPassword = EncryptDBPassword( $PlainPassword );

        $IniConfig->set( 'Connection', 'encpasswd', $EncPassword);
        $IniConfig->delete( 'Connection', 'passwd');
        $IniConfig->rewrite();
        DisconnectSite();

        # DatabaseConnectionData schould now return the password transparently,
        # which means unencrypted. We compare both the plain and encrypted text.
        ConnectToSite();
        $hSection = DatabaseConnectionData('Site');
        my $DecryptedPassword = $hSection->{'passwd'};
        #Restore.
        $IniConfig->set( 'Connection', 'passwd', $PlainPassword);
        $IniConfig->delete( 'Connection', 'encpasswd');
        $IniConfig->rewrite();
        DisconnectSite();

        Error("Not same password: '$DecryptedPassword' eq '$PlainPassword'")
          if $PlainPassword ne $DecryptedPassword;
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error;
        return 0;
    }
    return 1;
}

sub Initialize {
}

sub Cleanup {
}

1;

