use constant TESTS => 12;
use Test::More tests => TESTS;

use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectToSite DisconnectSite GetCurrentDBHandle );
use DE_EPAGES::Database::API::DBInterface qw( ProcessSqlGo );

RunScript(
    Debug => 1,
    Sub => sub {
        ConnectToSite();
        my $dbi = GetCurrentDBHandle();
        SKIP: {
            # skip 1 test if not Sybase
            skip( "No Sybase DBD driver\n", TESTS ) unless $dbi->driverName eq 'Sybase';

            $dbi = GetCurrentDBHandle();
            ok( $dbi, "GetCurrentDBHandle" );

            ok( Init( $dbi ), "Init" );
            ok( $dbi->existsTable('sysobjects'), 'sysobjects table exists' );

            ok( $dbi->getTableLockingScheme('dbitest') eq 'allpages', 'dbitest has allpages locking scheme' );

            $dbi->modifyTableLockingScheme('dbitest','datapages');
            ok( 1,  'set locking scheme of dbitest to datapages ' );

            ok( $dbi->getTableLockingScheme('dbitest') eq 'datapages', 'dbitest has allpages datapages scheme' );

            $dbi->modifyTableLockingScheme('dbitest','datapages') ;
            ok( 1,  'try to set locking scheme of dbitest to datapages again '  );

            ProcessSqlGo(<<END_SQL);
CREATE PROC sp_xxx AS
    -- comment
    SELECT 'xxx'
GO
go
CREATE PROC sp_yyy AS
    -- comment
    SELECT 'yyy'
END_SQL

            ok( $dbi->existsProcedure('sp_xxx'), "ProcessSqlGo: first statement" );
            ok( $dbi->existsProcedure('sp_yyy'), "ProcessSqlGo: second statement" );

            my $Text = $dbi->execute(<<END_SQL)->[0]->[0];
SELECT text
FROM syscomments sc
INNER JOIN sysobjects so ON sc.id = so.id
WHERE so.type = 'P'
  AND so.name = 'sp_xxx'
END_SQL
            ok( $Text =~ /-- comment/, "ProcessSqlGo: keep comments (EPG-14032)" );

            $dbi->dropProcedure('sp_yyy');
            $dbi->dropProcedure('sp_xxx');
            ok( !$dbi->existsProcedure('sp_yyy'), "dropProcedure" );

            ok( Cleanup( $dbi ), "Cleanup" );
        }
    }
);

DisconnectSite();

sub Init {
    my ( $dbi ) = @_;
    Cleanup( $dbi );

    $dbi->do( "CREATE TABLE dbitest( id INTEGER, name VARCHAR(50) NULL, dt DATETIME NULL )" );

    return $dbi->existsTableColumn( 'dbitest', 'name' );
}

sub Cleanup {
    my ( $dbi ) = @_;

    $dbi->do( "DROP TABLE dbitest" ) if $dbi->existsTable( 'dbitest' );
    $dbi->dropProcedure('sp_yyy') if $dbi->existsProcedure('sp_yyy');
    $dbi->dropProcedure('sp_xxx') if $dbi->existsProcedure('sp_xxx');;
    return 1;
}

1;

