use Test::More tests => 8;

use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectToSite DisconnectSite Transaction GetCurrentDBHandle );
use DE_EPAGES::Core::API::Error qw (Error);

RunScript(
    Debug => 1,
    Sub => sub {
        ConnectToSite();
        my $dbi = GetCurrentDBHandle();
        ok( $dbi, "GetCurrentDBHandle" );

        ok( Init( $dbi ), "Init" );

        $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
        Transaction(sub {
            $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);
            $dbi->do('insert into dbitest values (?,?)', [3,'2 in transaction']);
        });
        my $aaResults = $dbi->execute('select id from dbitest');
        ok (3 == scalar @$aaResults, 'transaction without error');
        $dbi->do('delete from dbitest');


        # error
        $aaResults = $dbi->execute('select id from dbitest');
        ok (0 == scalar @$aaResults, 'transaction with error - reset ');
        eval {
            $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
            Transaction(sub {
                $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);
                die('Transaction crash');
            });
        };
        $aaResults = $dbi->execute('select id from dbitest');
        ok (1 == scalar @$aaResults, 'transaction with delete error ');
        $dbi->do('delete from dbitest');


        # error
        $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
        $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);

        eval {
            Transaction(sub {
                $dbi->do('delete from dbitest');
                $dbi->do('insert into dbitest values (?,?)', [3,'2 in transaction']);
                die('Transaction crash');
            });
        };
        $aaResults = $dbi->execute('select id from dbitest');
        ok (2 == scalar @$aaResults, 'transaction with die');
        $dbi->do('delete from dbitest');

        # error
        eval {
            $dbi->do('insert into dbitest values (?,?)', [1,'before transaction']);
            Transaction(sub {
                $dbi->do('insert into dbitest values (?,?)', [2,'1 in transaction']);
                Transaction(sub {
                    $dbi->do('insert into dbitest values (?,?)', [3,'2 in transaction']);
                });
                Error('SoNicht');
            });
        };
        $aaResults = $dbi->execute('select id from dbitest');
        ok (1 == scalar @$aaResults, 'transaction with error');

        ok( Cleanup( $dbi ), "Cleanup" );
        DisconnectSite();
    }
);

DisconnectSite();

sub Init {
    my ( $dbi ) = @_;
    Cleanup( $dbi );

    $dbi->do( "CREATE TABLE dbitest( id INTEGER, name VARCHAR(50) NULL)" );

    return $dbi->existsTableColumn( 'dbitest', 'name' );
}

sub Cleanup {
    my ( $dbi ) = @_;

    $dbi->do( "DROP TABLE dbitest" ) if $dbi->existsTable( 'dbitest' );
    return 1;
}

1;

