/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.io.File $Revision: 1.19.52.1 $

*/
dojo.provide('epages.io.file');
dojo.require('epages.io.json');

epages.io.File = function() {
	this.files = [];
};

epages.io.File.prototype = {

	// boundary is hex presentation of timestamp
	boundary: (new Date()).getTime().toString(16),

	// supported encodings for file transfer, RAW should not be used, due to a bug in
	// the XMLHTTPRequest object
	encoding: {
		RAW:              undefined,
		BASE64:           'base64',
		QUOTEDPRINTABLE:  'quoted-printable',
		XHR:              'epages-xhr'
	},

	clear: function () {
		this.files = [];
	},

	addFile: function (/*String*/formElement, /*String*/fileName, /*String*/fileContent, /*String?*/encoding) {
		// summary:
		//    Adds a file for upload.
		// parameters:
		//    formElement: name of the form element
		//    fileName: name of the file
		//    fileContent: contents of the file;
		this.files.push({
			"file":             fileName,
			"name":             formElement,
			"content":          fileContent,
			"contentType":      "application/octet-stream; charset=utf-8",
			"transferEncoding": encoding != null ? this.encoding[encoding] : this.encoding.BASE64
		});
	},
	
	_getFilePost: function() {
		var post = '';
		for (var i = 0; i < this.files.length; i++) {
			var file = this.files[i];
			post += "--" + this.boundary + "\r\n"
						+ 'Content-Disposition: form-data; name="' + file.name + '"; filename="' + file.file + '"\r\n'
						+ 'Content-Type: ' + file.contentType + '\r\n';
			if (file.transferEncoding != '') {
				post += 'Content-Transfer-Encoding: ' + file.transferEncoding + "\r\n";
			}
			post += "\r\n" + file.content + "\r\n";
		}
		post += "--" + this.boundary + "--\r\n";
		return post;
	},

	send: function (/*String*/url) {
		// summary:
		//    uploads file via multipart post and returns the result document.
		// examples:
		//    fileUploader = new epages.io.File();
		//    fileUploader.addFile("NewFile", "test.txt", "This is a test.");
		//    result = fileUploader.send('http://www.mydomain.de/form.php');
                // return: String
		var outerData = {};
		dojo.rawXhrPost({
			file:       this.files,
			url:        url,
			sync:       true,
			multipart:  true,
			error:      epages.io.Json.prototype.defaultErrorCallback,
			load:       function(data){ outerData.data = dojo.fromJson(data); },
			headers:    { 
				'Accept':       'text/x-json',
				'Content-Type': 'multipart/form-data; boundary=' + this.boundary
			},
			postData:   this._getFilePost()
		});
		return outerData;
	}
};
