/* 
 * @charset			utf-8		(ÄÖÜ äöü ß)
 *
 * Copyright (c) 2006-2010, epages GmbH
 * All Rights Reservedojo.
 * epages.mimetype $Revision: 1.1.2.1 $
 */
 
(function(dojo) {
	dojo.provide('epages.mimetype');

	dojo.declare('epages._mimetype',null,{
	// summary: mimetype and filetype handling
	// description: 
		
		/*
		 * private
		 */
		 _MIMETYPE_MAP: {
			// image
			'image/png':																	['png' ],
			'image/jpeg':																	['jpe', 'jpeg', 'jpg' ],
			'image/gif':																	['gif' ],
			'image/bmp':																	['bmp' ],
			'image/vnd.microsoft.icon':														['ico' ],
			'image/tiff':																	['tif', 'tif' ],
			'image/svg+xml':																['svg', 'svgz' ],
			'image/x-win-bitmap':															['cur'],
			// audio/video
			'audio/mpeg':																	['mp3' ],
			'audio/mp4a-latm':																['m4p', 'm4b', 'm4a' ],
			'audio/x-wav':																	['wav' ],
			'audio/ogg':																	['ogg', 'oga', 'spx' ],
			'audio/flac':																	['flac' ],
			'audio/x-ms-wma':																['wmv'],
			'video/mp4':																	['mp4' ],
			'video/mpeg':																	['mpe', 'mpg', 'mpeg', 'mpga' ],
			'video/quicktime':																['qt', 'mov' ],
			'video/x-msvideo':																['avi' ],
			'video/x-flv':																	['flv' ],
			'video/x-ms-wmv':																['wmv'],
			// web
			'text/plain':																	['txt' ],
			'text/html':																	['htm', 'html', 'shtml', 'php', 'php4' ],
			'text/css':																		['css' ],
			'application/javascript':														['js' ],
			'application/json':																['json' ],
			'application/xml':																['xml', 'plist' ],
			'application/x-shockwave-flash':												['swf' ],
			// archives
			'application/zip':																['zip' ],
			'application/gzip':																['gz' ],
			'application/x-rar-compressed':													['rar' ],
			'application/x-msdownload':														['exe', 'msi' ],
			'application/vnd.ms-cab-compressed':											['cab' ],
			// adobe
			'application/pdf':																['pdf' ],
			'image/vnd.adobe.photoshop':													['psd' ],
			'application/postscript':														['ai', 'eps', 'ps' ],
			// ms office
			'application/msword':															['doc' ],
			'application/rtf':																['rtf' ],
			'application/vnd.ms-excel':														['xls' ],
			'application/vnd.ms-powerpoint':												['ppt' ],
			'application/vnd.openxmlformats-officedocument.wordprocessingml.document': 		['docx'],
			'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':			['xlsx']
		},
		
		_MIMETYPE_GROUPS: {
			'all':				['*/*' ],
			'icons':			['image/vnd.microsoft.icon', 'image/x-win-bitmap' ],
			'images':			['image/png', 'image/jpeg', 'image/gif' ],
			'documents':		['application/pdf', 'application/msword', 'application/vnd.ms-excel', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' ],
			'sounds':			['audio/mpeg', 'audio/x-ms-wma', 'audio/ogg', 'audio/flac', 'audio/x-ms-wma' ],
			'video':			['video/x-msvideo', 'video/mp4', 'video/mpeg', 'video/x-ms-wmv', 'video/x-flv' ]
		},
		
		_MIMETYPE_PREVIEWABLE: ['image/png', 'image/jpeg', 'image/gif', 'image/vnd.microsoft.icon' ],
		
		/*
		 * public
		 */	 
		
		/*
		 * init instance
		 */	
		constructor: function() {					
		},
		
		/*
		 * methods
		 */	
		mimeBySuffix: function( suffix , firstOnly ) {
			var ret = Array();
			var map = this._MIMETYPE_MAP;
			
			for( var i in map ) {
				if( dojo.indexOf(map[i], suffix)>=0 ) {
					if( firstOnly ) {
						ret = i;
						break;
					}
					else {
						ret.push(i);
					}
				}
			};
			
			return ret;
		},
		
		isGroup: function( mime , group ) {
			return group=='all' ? true : (dojo.indexOf(this._MIMETYPE_GROUPS[group] || [], mime)>=0 ? true : false);
		},
		
		isPreviewable: function( mime ) {
			return dojo.indexOf(this._MIMETYPE_PREVIEWABLE, mime)>=0 ? true : false;
		}
		
	});
	
	epages.mimetype = {
		mimeBySuffix: function( suffix , firstOnly ) {
			return new epages._mimetype().mimeBySuffix( suffix , firstOnly );
		},
		isPreviewable: function( mime ) {
			return new epages._mimetype().isPreviewable( mime );
		},
		isGroup: function( mime , group ) {
			return new epages._mimetype().isGroup( mime , group );
		},
		getGroup: function( group ) {
			var instance = new epages._mimetype();
			return instance._MIMETYPE_GROUPS[group] || instance._MIMETYPE_GROUPS['all'];
		},
		isGroupBySuffix: function( suffix , group ) {
			var ret = false,
				instance = new epages._mimetype(),
				mimes = instance.mimeBySuffix( suffix ),
				length = mimes.length;
				
			for( var i=0 ; i<length ; i++ ) {
				if( instance.isGroup( mimes[i] , group ) ) {
					ret=true;
					break;
				}
			}
				
			return ret;
		}
	};
	
})(dojo);