/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

  epages.string $Revision: 1.9.4.3 $

*/
dojo.provide("epages.string");
dojo.require('epages.epages');

epages.string = {

	escapeJavaScript: function (str) {
		return str.replace(/(["'\f\b\n\t\r])/gm, "\\$1");
	},

	escapeXml: function (str, noSingleQuotes) {
		str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
		if (!noSingleQuotes) {
			str = str.replace(/'/gm, "&#39;");
		}
		return str;
	},

	toBoolean: function(/*String*/value) {
		if (value === undefined) return false;
		if (typeof value == "boolean") return value;
		if (typeof value != "string") throw new Error("Not a string value, got " + typeof(value) + "!");
		return value.toLowerCase() === "true" || value == '1' ? true : false;
	},
	
	toInteger: function(/*String*/value,/*Boolean*/removeThousandDelimiter) {
	// summary: parse string to integer, optional remove thousandDelimiter
		if(removeThousandDelimiter) {
			value = value.replace(/\./g,'');
		}
		return parseInt( value.replace(/[^0-9\,\.]/g, "") );
	},

	toTime: function(/*String*/value, /*String*/epagesPattern) {
		if (value == '') return null;
		if (epagesPattern == null || epagesPattern.length == 0) epagesPattern = epages.vars.Locale.timeFormat;
		var pattern = epages.localize.getDojoDateTimePattern(epagesPattern);
		date = dojo.date.locale.parse(value, { selector: 'date', datePattern: pattern, timePattern: '' });
		if (date === null) date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'HH:mm a', timePattern: '' });
		if (date === null) date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'HH a', timePattern: '' });
		if (date === null) date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'HH', timePattern: '' });
		// allow entries like "24:00" which means "00:00"
		if (date === null) date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'KK:mm', timePattern: '' });
		if (date === null) date = dojo.date.locale.parse(value, { selector: 'date', datePattern: 'KK', timePattern: '' });
		return date;
	},

	cssSizeToNumber : function(/* string */ value) {
	// summary: convert css size to a number (float or integer)
		if(typeof value == "string") {
			if (value.match(/^([0-9]+)(px|pt)$/i)) {
				var n = parseInt(RegExp.$1);
				return n;
			}

			if (value.match(/^([0-9]*\.?[0-9]+)(pc|%|mm|cm|em|in|ex)$/i)) {
				var n = parseFloat(RegExp.$1);
				return n;
			}

			if (value.match(/^[0]+$/)) {
				return 0;
			};
		}
		return NaN;
	},

	// corrects a bug in IE's String.split, where IE discards all empty elements
	split: function(string, rx) {
		if (epages.Browser.engine != 'MSIE') return string.split(rx);
		var unique = '';
		do {
			unique += Date.toString();
		} while(string.match(unique));
		string = string.replace(rx, '|' + unique + '|');
		var splits = string.split(unique);
		for (var i = 0; i < splits.length; i++) {
			splits[i] = splits[i].replace(/^(?:\|)?(.*?)(?:\|)?$/g, function(tmp, match) { return match } );
		}
		return splits;
	}
};

epages.string.Base64 = function () {
}

epages.string.Base64.prototype = {
	// private property
	_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

	// public method for encoding
	encode : function (input) {
		var output = "";
		var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
		var i = 0;

		input = this._utf8_encode(input);
		
		if (typeof(window.btoa) == 'function') {
			return window.btoa(input);
		}

		while (i < input.length) {

			chr1 = input.charCodeAt(i++);
			chr2 = input.charCodeAt(i++);
			chr3 = input.charCodeAt(i++);

			enc1 = chr1 >> 2;
			enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
			enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
			enc4 = chr3 & 63;

			if (isNaN(chr2)) {
				enc3 = enc4 = 64;
			} else if (isNaN(chr3)) {
				enc4 = 64;
			}

			output = output +
			this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
			this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

		}

		return output;
	},

	// public method for decoding
	decode : function (input) {
		var output = "";
		var chr1, chr2, chr3;
		var enc1, enc2, enc3, enc4;
		var i = 0;

		input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
		
		if (typeof(window.atob) == 'function') {
			output = window.atob(input);
		} else {

			while (i < input.length) {
	
				enc1 = this._keyStr.indexOf(input.charAt(i++));
				enc2 = this._keyStr.indexOf(input.charAt(i++));
				enc3 = this._keyStr.indexOf(input.charAt(i++));
				enc4 = this._keyStr.indexOf(input.charAt(i++));
	
				chr1 = (enc1 << 2) | (enc2 >> 4);
				chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
				chr3 = ((enc3 & 3) << 6) | enc4;
	
				output = output + String.fromCharCode(chr1);
	
				if (enc3 != 64) {
					output = output + String.fromCharCode(chr2);
				}
				if (enc4 != 64) {
					output = output + String.fromCharCode(chr3);
				}
	
			}
		}

		output = this._utf8_decode(output);

		return output;

	},

	// private method for UTF-8 encoding
	_utf8_encode : function (string) {
		string = string.replace(/\r\n/g,"\n");
		var utftext = "";

		for (var n = 0; n < string.length; n++) {

			var c = string.charCodeAt(n);

			if (c < 128) {
				utftext += String.fromCharCode(c);
			}
			else if((c > 127) && (c < 2048)) {
				utftext += String.fromCharCode((c >> 6) | 192);
				utftext += String.fromCharCode((c & 63) | 128);
			}
			else {
				utftext += String.fromCharCode((c >> 12) | 224);
				utftext += String.fromCharCode(((c >> 6) & 63) | 128);
				utftext += String.fromCharCode((c & 63) | 128);
			}

		}

		return utftext;
	},

	// private method for UTF-8 decoding
	_utf8_decode : function (utftext) {
		var string = "";
		var i = 0;
		var c = c1 = c2 = 0;

		while ( i < utftext.length ) {

			c = utftext.charCodeAt(i);

			if (c < 128) {
				string += String.fromCharCode(c);
				i++;
			}
			else if((c > 191) && (c < 224)) {
				c2 = utftext.charCodeAt(i+1);
				string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
				i += 2;
			}
			else {
				c2 = utftext.charCodeAt(i+1);
				c3 = utftext.charCodeAt(i+2);
				string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
				i += 3;
			}
		}
		return string;
	}
}
