﻿/*
	Copyright (c) 2006-2010, epages Software GmbH
	All Rights Reserved.

	epages.upload $Revision: 1.1.2.20.2.3 $
*/

dojo.provide("epages.uploader");
dojo.require("epages.widget.Modaldialog");
dojo.require("epages.event.interactionrestrictor");
dojo.require("dojo.fx");
dojo.require("dojox.fx");
dojo.require("dojo.fx.easing");
dojo.require('epages.template');
dojo.require("epages.io.translation");
dojo.require("epages.uimessagehandler");

epages.Uploader = {
	// HTML skeleton snippet to be inserted into the upload dialog
	_DIALOG_TEMPLATE:
'<div class="Uploader">\
		<div class="FileListContainer"><div class="HideElement"></div></div>\
		<table class="ContentList UploadAddLinkContainer">\
			<tbody>\
				<tr class="NewEntry">\
					<td class="AlignCenter" colspan="2">\
						<span class="UploadAddLink DoActionLink EnableHover">\
							<span class="BigUploadLink">\
								<i class="Sprite (%add_button_class%)"></i>\
								<span>(%text_choose_files%)</span>\
							</span>\
							<span class="HideElementSoft SmallUploadLink">\
								<i class="Sprite (%add_button_class%)"></i>\
								<span>(%text_add_more_files%)</span>\
							</span>\
						</span>\
					</td>\
				</tr>\
			</tbody>\
		</table>\
		<div class="UploadFormContainer HideElement"></div>\
	</div>\
	<div class="DialogMessage MessageInfo TopMargin HideElement"></div>\
</div>',

	// template parsed version of _DIALOG_TEMPLATE
	_PARSED_DIALOG_TEMPLATE: 					"",

	// translate string constants
	_DIALOG_TITLE:										"",
	//_SELECT_LINK_LABEL:								"",
	//_SELECT_LINK_MORE_LABEL:							"",
	_UPLOAD_BUTTON_LABEL:								"",
	_CANCEL_BUTTON_LABEL:								"",
	//_CLOSE_BUTTON_LABEL:								"",

	// TODO: translate error codes
	_ERROR_HEADLINE:									"",
	_MAXCONTENTLENGTH_EXCEEDED_ERROR: 					"",
	_UPLOADSPACE_EXCEEDED_ERROR:      					"",
	_SESSION_ERROR:										"",
	_MAX_PREVIEW_ITEMS:									10,
	_ENABLE_PREVIEW:									false,

	// symbols to be displayed equating to the file extension (type)
	_FILE_EXTENSION_ICON_MAP: {

		ICO:									"ico_l_image",
		PNG:									"ico_l_image",
		JPG:									"ico_l_image",
		JPEG:									"ico_l_image",
		BMP:									"ico_l_image",
		GIF:									"ico_l_image",

		WAV:									"ico_xl_box",
		MP3:									"ico_xl_box",
		MP4:									"ico_xl_box",
		WMA:									"ico_xl_box",
		AU:										"ico_xl_box",
		OGG:									"ico_xl_box",
		FLAC:									"ico_xl_box",

		AVI:									"ico_xl_file_avi",
		WMV:									"ico_xl_file_wmv",
		FLV:									"ico_xl_file_swf",
		MPG:									"ico_xl_file_avi",
		MPEG:									"ico_xl_file_avi",
		MP4:									"ico_xl_file_avi",
		SWF:									"ico_xl_file_swf",
		RM:										"ico_xl_file_rm",
		RAM:									"ico_xl_file_ram",
		DCR:									"ico_xl_file_dcr",
		MOV:									"ico_xl_file_mov",
		QT:										"ico_xl_file_mov",
		OGV:									"ico_xl_file_mov",

		DOC:									"ico_xl_box",
		DOT:									"ico_xl_box",
		XLS:									"ico_xl_box",
		XLA:									"ico_xl_box",
		PPT:									"ico_xl_box",
		PDF:									"ico_xl_file_pdf",
		HTM:									"ico_xl_file_htm",
		HTML:									"ico_xl_file_htm",

		TXT:									"ico_xl_box",
		CSS:									"ico_xl_box",
		JS:										"ico_xl_box",
		AS:										"ico_xl_box",

		ZIP:									"ico_m_folder",
		RAR:									"ico_m_folder",
		TAR:									"ico_m_folder",
		GZ:										"ico_m_folder",

		UNKNOWN:								"ico_xl_box"
	},

	// file extensions which can be previed via <img/>
	_fileExtensionsPreviewable: {
		ICO:									true,
		PNG:									true,
		GIF:									true,
		JPG:									true,
		JPEG:									true
	},

	// file extension / type map useful for limiting the file selection dialog
	// only for future use - browsers ignore the "accept" attribute on file inputs by now
	// TODO: find a way to limit the file selection with some sort of type filter
	_fileTypes: {
		Icons:								["ICO", "CUR"],
		Images:								["JPG", "JPEG", "GIF", "PNG"],
		Documents:						["PDF", "DOC", "RTF", "TXT"],
		Sound:								["MP3", "MP4", "WMA", "WAV", "FLAC", "OGG"],
		Video:								["AVI", "WMV", "FLV", "MPG", "MPEG", "MP4", "OGV"]
	},

	// top part of the file list table
	_fileListTopHTML:				"<table class=\"ContentList FileListTable\"><tbody class=\"FileList\">",
	// bottom part of the file list table
	_fileListBottomHTML:		"</tbody></table>",
	// middle part of the file list table represented as array (for the use of join("") later on)
	_fileListHTMLSnippets:	[''],

	// reference to the upload dialog
	_uploadDialog:					null,

	// reference to the node which holds the file list (and the upload <form/> for browsers without binary xhr support)
	_uploadFormContainer:		null,
	// reference to the "add more" link
	_uploadAddLink:					null,
	// reference to the parent node of the "add more" link
	_uploadAddLinkContainer:null,
	// reference to the file list parent
	_fileListContainer:			null,
	// reference to the <input type="file"/> used to select the files
	_uploadInput:						null,

	// array of all files inside the queue
	_uploadCollection:			[],

	// array of all upload responses
	_uploadResponseUploadedFiles:			[],

	// array of all files inside the error msg
	_uploadErrorFiles: [],

	// index of the current file to be uploaded
	_uploadIndex:						0,

	// settings of the current Uploader
	_uploadSettings:				{},

	// flags if the uploader is currently in use
	_uploadActive:					false,
	// flags if errors occured on uploading process
	_uploadErrors:					false,

	// equates to "multiple" flag on "settings" object aside from browsers which don't support multiple uploads
	_multipleFileInputEnabled: false,

	// estimated time for a single upload process (will be calculated precisely later on)
	_uploadTimeEstimation:	200,
	// date object used to calculate the estimated upload time
	_uploadOldTime:					null,
	// reference to the node which represents the upload status in percent
	_uploadOldPercentageBar:null,
	// reference the the dojo animation used to smoothly represent the upload status
	_statusBarAni:					null,

	_resetUploadSettings: function(uploadSettings /* Object */) {
		// summary:
		//	set the default settings object for this instance
		// description:
		//	this is called on startup too to initially fill the settings object

		uploadSettings = uploadSettings || {};
		this._uploadSettings = {
			action:					uploadSettings.action || "?ChangeAction=UploadFiles&ObjectID=" + epages.vars.ObjectID,
			directory:			uploadSettings.directory || null,
			multiple:				uploadSettings.multiple ? epages.string.toBoolean(uploadSettings.multiple) : false,
			fileType:				uploadSettings.fileType || "",
			reloadPage:			uploadSettings.reloadPage ? epages.string.toBoolean(uploadSettings.reloadPage) : false,
			callback:				uploadSettings.callback || function() {},
			formElement:		uploadSettings.formElement || "NewFile",
			id: 						uploadSettings.id || "uploadAppletNodeID"
		};

		// fix any "amp;"s
		this._uploadSettings.action = this._uploadSettings.action.replace(/&amp;/,'&');

		// we need this in every case since there is no use of text/x-json as a result of a <form/>.submit()
		if(!this._uploadSettings.action.match(/handleashtml/)) {
			this._uploadSettings.action += "&handleashtml=1";
		}

		// default changeaction
		if(!this._uploadSettings.action.match(/ChangeAction/)) {
			this._uploadSettings.action += "&ChangeAction=UploadFiles";
		}

		// default for viewaction
		if(!this._uploadSettings.action.match(/ViewAction/)) {
			this._uploadSettings.action += "&ViewAction=JSONViewResponse";
		}

		if(this._uploadSettings.directory) {
			this._uploadSettings.action += "&Directory=" + this._uploadSettings.directory;
		}
	},

	_showUploadDialog: function() {
		// summary:
		//	wrapper around the native show method of the modaldialog widget

		if(this._uploadDialog) {
			this._uploadDialog.show();
		}
	},

	_hideUploadDialog: function() {
		// summary:
		//	wrapper around the native hide method of the modaldialog widget

		if(this._uploadDialog) {
			this._uploadDialog.hide();
		}
	},

	_maximizeUploadArea: function() {
		// summary:
		//	show the big file selection link (NewEntry area)

		dojo.removeClass(dojo.query(".BigUploadLink", this._uploadAddLink)[0], "HideElementSoft");
		dojo.addClass(dojo.query(".SmallUploadLink", this._uploadAddLink)[0], "HideElementSoft");
	},

	_minimizeUploadArea: function() {
		// summary:
		//	show the tiny file selection link (NewEntry area)

		dojo.addClass(dojo.query(".BigUploadLink", this._uploadAddLink)[0], "HideElementSoft");
		dojo.removeClass(dojo.query(".SmallUploadLink", this._uploadAddLink)[0], "HideElementSoft");
	},

	_cleanUp: function(uploadSettings) {
		// summary:
		//	reset all predefined variables
		// description:
		//	this emulates the constructor since epages.Uploader is called and not instantiated via "new"
		uploadSettings = uploadSettings || this._uploadSettings;

		this._resetUploadSettings(uploadSettings);
		if(this._uploadFormContainer) {
			this._fileListHTMLSnippets = [''];
			this._uploadErrorFiles = [];
			this._uploadCollection = [];
			this._uploadResponseUploadedFiles = [];
			this._uploadIndex = 0;
			this._uploadActive = false;
			this._uploadErrors = false;
			this._uploadFormContainer.innerHTML = this._fileListContainer.innerHTML = "<div class=\"HideElement\"></div>";
			dojo.removeClass(this._fileListContainer.parentNode, "FilesSelected");
			dojo.removeClass(this._uploadAddLinkContainer, "HideElement");
			dojo.removeClass(this._uploadDialog.innerButtonBarNode.firstChild, "Active");
			dojo.removeClass(this._uploadDialog.innerButtonBarNode.firstChild, "Disabled");
			this._maximizeUploadArea();
		}
	},

	_onSubmitUploadDialog: function(evt) {
		// summary:
		//	start the uploading process (mapped to "click" event)

		if(dojo.hasClass(evt.currentTarget, "Disabled")) {
			return;
		}

		dojo.removeClass(this._uploadDialog.innerButtonBarNode.firstChild, "Active");
		dojo.addClass(this._uploadDialog.innerButtonBarNode.firstChild, "Disabled");
		dojo.addClass(this._uploadAddLinkContainer, "HideElement");

		this._uploadOldTime = new Date();
		this._uploadActive = true;
		this._uploadNextQueueItem();
	},

	_uploadNextQueueItem: function() {
		// summary:
		//	recursively upload one file after another

		this._updateProgressDisplay();

		// we're uploading the file list in reversed order (top > bottom)
		var file = this._uploadCollection[this._uploadCollection.length - 1 - this._uploadIndex];

		if(this._multipleFileInputEnabled) {
			// xhr upload
			var xhr = new XMLHttpRequest(); // a plain xhr object because dojo can't handle binary requests at all
			xhr.open("POST", this._uploadSettings.action);
			xhr.setRequestHeader("Content-Type", "application/octet-stream");
			xhr.setRequestHeader("If-Modified-Since", "Mon, 26 Jul 1997 05:00:00 GMT");
			xhr.setRequestHeader("Cache-Control", "no-cache");

			var contentDisposition = "attachment; ";
			contentDisposition += "filename=\"" + encodeURIComponent(file.fileName || file.name)  + "\"";
			var formElementName = this._uploadSettings.formElement;
			if(formElementName) {
				contentDisposition += "; name=\"" + encodeURIComponent(formElementName)  + "\"";
			}
			xhr.setRequestHeader("content-disposition", contentDisposition);

			if(xhr.sendAsBinary) {
				xhr.sendAsBinary(file.getAsBinary());
			}else {
				xhr.send(file);
			}
			dojo.connect(xhr, "readystatechange", function() { // listening to "onprogress" would be better but is very / too hard to normalize by now
				if(xhr.readyState === 4) {
					epages.Uploader._onResponse(xhr.responseText);
				}
			});
			dojo.connect(xhr, "error", this, "_onResponse");
		}
		else {
			// "classic" upload via <form/>.submit()
			this._uploadFormContainer.innerHTML = "";

			var fileUploadForm = document.createElement("form");
			fileUploadForm.action = this._uploadSettings.action;
			fileUploadForm.method = "POST";
			fileUploadForm.enctype = fileUploadForm.encoding = "multipart/form-data";
			fileUploadForm.target = "fileUploadFrame";
			this._uploadFormContainer.appendChild(fileUploadForm);

			var uploadFrameContainer = document.createElement("div");
			uploadFrameContainer.innerHTML = "<iframe name=\"fileUploadFrame\" onload=\"epages.Uploader._onResponse(this.contentWindow.document.body.innerHTML)\"></iframe>";
			this._uploadFormContainer.appendChild(uploadFrameContainer);

			fileUploadForm.appendChild(file);

			fileUploadForm.submit();
		}
	},

	_updateProgressDisplay: function() {
		// summary:
		//	making the uploading process endurable while showing some nice animations

		dojo.publish("interactionRestrictor/permit", ["epages_Uploader_restrictor"]);

		var tdNode = this._fileListContainer.getElementsByTagName("tr")[this._uploadIndex].firstChild;
		dojo.publish("interactionRestrictor/suspend", ["epages_Uploader_restrictor", tdNode]);
		// position calculation inside scrollable elements is wrong...we have to correct it manually
		var restrictor = $("epages_Uploader_restrictor");
		var loaderAni = $("LoaderAniNode");
		restrictor.style.top = (restrictor.offsetTop - this._fileListContainer.scrollTop + 2) + "px";
		loaderAni.style.top = (loaderAni.offsetTop - this._fileListContainer.scrollTop + 2) + "px";

		var statusBar = dojo.query("div.StatusBar", tdNode.parentNode)[0];
		var percentageBar = dojo.query("div.InnerStatusBar > div", statusBar)[0];

		if(this._statusBarAni) {
			this._statusBarAni.stop();
			this._uploadOldPercentageBar.style.width = (statusBar.offsetWidth - 2) + "px";
		}

		this._statusBarAni = dojox.fx.wipeTo({node: percentageBar, width: statusBar.offsetWidth - 2, duration: this._uploadTimeEstimation, easing: dojo.fx.easing.quadIn});
		this._statusBarAni.play();

		var currentDate = new Date();
		this._uploadTimeEstimation = ~~((currentDate - this._uploadOldTime) / 2);
		this._uploadOldTime = currentDate;
		this._uploadOldPercentageBar = percentageBar;

	},

	_throwError: function(errorText /* String */, trEntry /* <tr/> Node */) {
		// summary:
		//	an error occured for a specific upload item

		this._uploadErrors = true;
		var percentageBar = dojo.query(".StatusBar div.InnerStatusBar div", trEntry)[0];
		percentageBar.className = "PercentageCritical";

		// TODO: show some sort of error message
		top.dojo.publish("uimessage/show", [ "", errorText, "Dialog" , {titleBar: this._ERROR_HEADLINE, typeClass: "Error", sizeClass:"Large"}]);
	},

	_onResponse: function(responseSnippet /* ? */) {
		// summary:
		//	upload finished for a specific file and we (maybe) received a server response

		responseSnippet = responseSnippet || ""; // be sure that it will never be undefined
		var response;

		var trEntry = this._fileListContainer.getElementsByTagName("tr")[this._uploadIndex];

		try {
			// there is a chance that we receive garbage which can't be transformed into an JSON object
			// with that in mind there is no way around the try/catch statement
			response = eval("[" + responseSnippet + "]")[0];
		}catch(e) {}

		if(!response) {
			// hard error
			dojo.addClass(trEntry, "UploadError");
			if(responseSnippet.match("file is too large")) {
				this._throwError(this._MAXCONTENTLENGTH_EXCEEDED_ERROR, trEntry);
			}else {
				//outdated session, false token, something else happened
				this._throwError(this._SESSION_ERROR, trEntry);
			}
			this._uploadActive = false; // stop polling
			dojo.publish("interactionRestrictor/permit", ["epages_Uploader_restrictor"]);
		} else if(response.Errors) {
			// soft error
			if(response.Errors[0].Message == "FeatureLimitExceeded") {
				this._throwError(this._UPLOADSPACE_EXCEEDED_ERROR, trEntry);
			} else {
				this._throwError(response.Errors[0].Message, trEntry);
			}
			dojo.addClass(trEntry, "UploadError");
			this._uploadActive = false; // stop polling
			dojo.publish("interactionRestrictor/permit", ["epages_Uploader_restrictor"]);
		} else {
			// success
			dojo.addClass(trEntry, "UploadSuccess");

			if( response.uploadedFiles ) {
				this._uploadResponseUploadedFiles.push(response.uploadedFiles);
			}

			this._uploadIndex++;
			if(this._uploadIndex < this._uploadCollection.length && this._uploadActive) {
				this._uploadNextQueueItem();
			}else {
				this._onUploadFinished(response);
			}
		}

		if(trEntry.nextSibling) {
			this._fileListContainer.scrollTop = trEntry.nextSibling.offsetTop - this._fileListContainer.offsetHeight + trEntry.nextSibling.offsetHeight;
		}

	},

	_onUploadFinished: function(response) {
		// summary:
		//	all files are uploaded - now show the user the results and sum it up
		dojo.publish("interactionRestrictor/permit", ["epages_Uploader_restrictor"]);
		if(this._uploadErrors) {
			// TODO: show some sort of error message to sum it up and provide a button for closing the dialog the soft way (not via "abort");
		} else {
			this._hideUploadDialog();
			this._uploadSettings.callback();
			if(this._uploadSettings.reloadPage) {
				var dialogArea = actualDialogArea ? "DialogArea=" + actualDialogArea : "";
				if(location.href.search(/Monitor/) != -1 && epages.vars.BaseUrl && epages.vars.ViewAction && epages.vars.ObjectID){
					var newHref = epages.vars.BaseUrl + '?ViewAction=' + epages.vars.ViewAction + '&ObjectID=' + epages.vars.ObjectID;
					if(dialogArea == ''){
						location.href = newHref;
					}else{
						location.href = newHref + '&'+dialogArea;
					}
				}
				else{
					if(location.href.search(/DialogArea/) != -1){
						var newLocation;
						if (this.useBaseUrlForHref) {
							newLocation = epages.vars.BaseUrl + '?ViewAction=' + epages.vars.ViewAction + '&ObjectID=' + epages.vars.ObjectID;
						} else {
							newLocation = location.href;
						}
						for(var i = 0; i < epagesDialog.areaName.length; i++){
							var regEx = new RegExp('DialogArea='+epagesDialog.areaName[i]);
							newLocation = newLocation.replace(regEx,dialogArea);
						}
						location.href = newLocation;
					}else{
						if(dialogArea == ''){
							location.href = location.href;
						}else {
							location.href = location.href + '&'+dialogArea;
						}
					}
				}
			}

			response.uploadedFiles = this._uploadResponseUploadedFiles;

			dojo.publish(this._uploadSettings.id + '/onUploadFinished', [ response ]);
		}
	},

	_onCancelUploadDialog: function() {
		// summary:
		//	user pressed "cancel" on the upload dialog

		this._uploadActive = false;
		this._hideUploadDialog();
	},


	_getFileExtension: function(fileAlias /* String */) {
		// summary:
		//	extract the file extension from a file name

		// TODO: insufficient > what is with files like this: "bla.blabla.blubb.jpg"?
		var arr = fileAlias.split(".");
		return arr[1] ? arr[1].toUpperCase() : '';
	},

	_getSymbolSrcForFile: function(fileAlias /* String */) {
		// summary:
		//	get a symbol representing the file type

		return this._FILE_EXTENSION_ICON_MAP[this._getFileExtension(fileAlias)] || this._FILE_EXTENSION_ICON_MAP.UNKNOWN;
	},

	_addFileListRow: function(fileAlias /* String */, fileIconSrc /* String */, isSprite /* Boolean */) {
		// summary:
		//	add a representation of a selected file to the file list

		var snippet = ["<tr" + ((this._fileListHTMLSnippets.length % 2) ? " class=\"alternate\"" : "" ) + "><td class=\""];
		var isPreviewAble = false;
		if(this._ENABLE_PREVIEW && this._fileListHTMLSnippets.length < this._MAX_PREVIEW_ITEMS && this._fileExtensionsPreviewable[this._getFileExtension(fileAlias)]) {
			isPreviewAble = true;
		}

		if(isPreviewAble && !isSprite) {
			snippet.push("CursorPointer\" onclick=\"showImage(this.firstChild.src)\"><img src=\"" + fileIconSrc + "\"/>");
		}else {
			snippet.push("\"><i class=\"Sprite NoPadding " + this._getSymbolSrcForFile(fileAlias)+ "\"></i>");
		}
		snippet.push("</td><td>" + fileAlias + "<div class=\"StatusBar\"><div class=\"InnerStatusBar\"><div class=\"PercentageNormal\" style=\"width:0\"></div></div></div></td></tr>");
		this._fileListHTMLSnippets.push(snippet.join(""));
	},

	_addFileToUploadQueue: function(file /* file binary */) {
		// summary:
		//	attach a file to be uploaded to the internal upload list

		this._uploadCollection.push(file);
	},

	_appendToQueue: function(fileInput /* <input type="file"/> */) {
		// summary:
		//	queue one or more files selected via <input type="file"/>

		// only one file may be selected but the user selected another one
		// so we delete the previous file selection
		if(!this._uploadSettings.multiple) {
			this._cleanUp();
		}

		dojo.addClass(this._fileListContainer.parentNode, "FilesSelected");

		if(fileInput.files) {
			this._multipleFileInputEnabled = true;
			dojo.forEach(fileInput.files, function(file, fileIndex) {
				isPreviewAble = (this._ENABLE_PREVIEW && (this._fileListHTMLSnippets.length < this._MAX_PREVIEW_ITEMS) && this._fileExtensionsPreviewable[this._getFileExtension(file.fileName)]);
				this._addFileListRow(file.fileName, (isPreviewAble && file.getAsDataURL) ? file.getAsDataURL() : this._getSymbolSrcForFile(file.fileName), (isPreviewAble && file.getAsDataURL) ? false : true);
				this._addFileToUploadQueue(file);
			}, this);
		}
		else {
			var fileAlias = fileInput.value;
			if(fileAlias.match(":")) {  // i.e. "C:\"
				// the local path is part of the fileAlias (Windows fake path, i.e. "C:\fakepath\ico_64hover_featurepacks_orgRGB.png").
				// We only want the name of the file: (C:\fakepath)[\](ico_64hover_featurepacks_orgRGB.png)
				fileAlias = fileAlias.match(/(.*)[\/\\]([^\/\\]+\.\w+)$/)[2];
			}
			this._addFileListRow(fileAlias, this._getSymbolSrcForFile(fileAlias), true);
			this._addFileToUploadQueue(fileInput);
		}

		this._fileListContainer.innerHTML = this._fileListTopHTML + this._fileListHTMLSnippets.join("") + this._fileListBottomHTML;
		this._minimizeUploadArea();
		this._insertUploadInput();
	},

	_queueFiles: function() {
		// summary:
		//	queue one or more files selected via <input type="file"/> with a certain delay and hide the upload dialog beforehand
		this._hideUploadDialog();
		setTimeout(
			dojo.hitch(
				this,
				function() {
					// TODO: _appendToQueue can take a long a time with > 50 files selected at once - we need some sort of becalmer animation for the user
					this._appendToQueue(this._uploadInput);
					setTimeout(
						dojo.hitch(
							this,
							function() {
								this._showUploadDialog();
								this._fileListContainer.scrollTop = 0;
							}
						),
						this._uploadDialog.duration
					);
				}
			),
			this._uploadDialog.duration + 10
		);
	},

	_insertUploadInput: function() {
		// summary:
		//	create a <input type="file"/>
		// description:
		//	the created file input will reside on top of the "add more" link on which the user will think he clicks on
		if(this._uploadInput) {
			this._uploadInput.parentNode.removeChild(this._uploadInput);
		}
		this._uploadInput = document.createElement("input");
		this._uploadInput.type = "file";
		this._uploadInput.name = this._uploadSettings.formElement;
		if(this._uploadSettings.multiple) {
			// this._uploadInput.multiple = true < won't work
			this._uploadInput.setAttribute("multiple", "true");
		}
		this._uploadInput.className = "UploadInputOverlay";
		this._uploadAddLink.appendChild(this._uploadInput);
		dojo.connect(this._uploadInput, "change", this, "_queueFiles");
	},

	_buildDialog: function() {
		// summary:
		//	build up the upload dialog

		var dlgParams = {
			title : this._DIALOG_TITLE,
			width : 450,
			scroll: "hidden",
			content: this._PARSED_DIALOG_TEMPLATE,
			buttons: [{
				label: this._UPLOAD_BUTTON_LABEL,
				onclick: dojo.hitch(this, this._onSubmitUploadDialog)
			},{
				label: this._CANCEL_BUTTON_LABEL,
				onclick: dojo.hitch(this, this._onCancelUploadDialog)
			}]
		};

		if(this._uploadDialog) {
			// update dialog content
			this._fileListContainer = undefined;
			this._uploadDialog.setupContent(dlgParams);
		}
		else {
			this._uploadDialog = new epages.widget.Modaldialog({
				options:{
					title : dlgParams.title,
					width : dlgParams.width,
					scroll: dlgParams.scroll,
					content: dlgParams.content
				},
				buttons: dlgParams.buttons
			});
			dojo.body().appendChild(this._uploadDialog.domNode);
		}
	},

	_mapNodes: function() {
		// summary:
		//	build up references the the importand and most used dialog nodes

		if(this._fileListContainer) {
			return;
		}
		var innerNode = this._uploadDialog.domNode;
		this._fileListContainer = dojo.query(".FileListContainer", innerNode)[0];
		this._uploadFormContainer = dojo.query(".UploadFormContainer", innerNode)[0];
		this._uploadAddLink = dojo.query(".UploadAddLink", innerNode)[0];
		this._uploadAddLinkContainer = dojo.query(".UploadAddLinkContainer", innerNode)[0];
	},

	_applyTranslations: function() {
		// summary:
		//	replaces member variables with caption texts with its correct translation strings
		var translation = epages.io.dictionary.translation;
		var uploadText = this._uploadSettings.multiple ? translation.get("FileUploadMultiple") : translation.get("FileUploadSingle");

		var templatedDialog = $TMPL(this._DIALOG_TEMPLATE, {
			add_button_class 		: 'ico_s_add',
			text_choose_files 	: uploadText+'...',
			text_add_more_files : this._uploadSettings.multiple ? translation.get("AddMoreImagesLink") : translation.get("AddMoreImagesLink")
		}, false);
		this._PARSED_DIALOG_TEMPLATE = templatedDialog.generateOutput();

		this._DIALOG_TITLE = uploadText;
		this._UPLOAD_BUTTON_LABEL = uploadText;
		this._CANCEL_BUTTON_LABEL = translation.get("Cancel");
		this._MAXCONTENTLENGTH_EXCEEDED_ERROR = translation.get("FileSizeLimitError");
		this._UPLOADSPACE_EXCEEDED_ERROR = translation.get("FeatureLimitExceeded");
		this._SESSION_ERROR = translation.get("InternalServerError");
		this._ERROR_HEADLINE = translation.get("FileUploadError");
	},

	open: function(uploadSettings /* Object */) {
		// summary:
		//	open an upload dialog
		// example:
		//	onclick="epages.Uploader.open({multiple:true, directory:'#Directory', reloadPage:true})"

		// only one Uploader dialog at once may be active
		// and we have to assume that is was opened before
		this._hideUploadDialog();
		this._cleanUp(uploadSettings);
		this._applyTranslations();
		this._buildDialog();
		this._mapNodes();
		this._insertUploadInput();
		this._maximizeUploadArea();
		this._showUploadDialog();
	}

};