/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.widget.ContentList $Revision: 1.3.4.1 $

*/

dojo.provide('epages.widget.ContentList');

dojo.require('epages.epages');
dojo.require('epages.event');
dojo.require('epages.browser');

/*
 add the class "NoRowSelector" to your element if you do not want to trigger the row selection for that specific element
 add the class "RowSelector" to your element if you explicitly want to trigger the row selection (even the element is an image / form element / whatever)
 add the class "NoSmallTableMode" to your table if the table is not meant to be displayed in this mode
*/

dojo.declare("epages.widget.ContentList", [dijit._Widget], {

	_getRelevantInputCell: function(tableRow) {
		var cellsInRow = tableRow.cells;
		for(var i = 0; i < cellsInRow.length; i++) {
			var inputsInCell = cellsInRow[i].getElementsByTagName("input");
			for(var j = 0; j < inputsInCell.length; j++) {
				if(inputsInCell[j] && inputsInCell[j].type != "hidden") {
					return cellsInRow[i];
				}
			}
		}
	},

	_calcCellWidth: function(cell) {
		if(cell.parentNode.parentNode.parentNode.className.match("ContentListForcedWidth")) {
			var tableHead = cell.parentNode.parentNode.parentNode.tHead.rows[0];
			var tableHeadCell = tableHead.cells[cell.cellIndex];
			if(!(tableHeadCell.className || "").match("Checkbox") && !(tableHeadCell.className || "").match("NoPadding") && tableHead.currentCellIndex != cell.cellIndex) {
				tableHead.currentCellIndex = cell.cellIndex;
				var parentWidth = tableHead.parentNode.parentNode.parentNode.offsetWidth;
				var tableWidth = tableHead.parentNode.parentNode.offsetWidth;
				tableWidth = ((parentWidth < tableWidth) ? parentWidth : tableWidth);
				var tableColumns = tableHead.cells.length;
				for(var i = 0; i < tableColumns; i++) {
					if(!(tableHead.cells[i].className || "").match("Checkbox")) {
						tableHead.cells[i].style.width = (tableWidth / tableColumns / 1.25) + "px";
					}
				}
				tableHeadCell.style.width = (tableWidth / tableColumns * 2) + "px";
			}
		}
	},

	_enableSmallTableMode: function(tableRows) {
		for(var i = 0; i < tableRows.length; i++) {
			for(var j = 0; j < tableRows[i].cells.length; j++) {
				dojo.connect(tableRows[i].cells[j], "mouseenter", this, function(evt) {
					this._calcCellWidth(evt.currentTarget);
				});
				dojo.connect(tableRows[i].cells[j], "mouseleave", this, function(evt) {
					var cell = evt.currentTarget;
					if(cell.parentNode.parentNode.parentNode.className.match("ContentListForcedWidth")) {
						var tableHead = cell.parentNode.parentNode.parentNode.tHead.rows[0];
						var tableHeadCell = tableHead.cells[cell.cellIndex];
						if(!(tableHeadCell.className || "").match("Checkbox") && !(tableHeadCell.className || "").match("NoPadding") && tableHead.currentCellIndex != cell.cellIndex) {
							tableHead.currentCellIndex = cell.cellIndex;
							for(var i = 0; i < tableHead.cells.length; i++) {
								if(!(tableHead.cells[i].className || "").match("Checkbox")) {
									tableHead.cells[i].style.width = "auto";
								}
							}
						}
					}
				});
			}
		}
	},

	_setCheckTableMode: function() {
		var tRows = this.domNode.tBodies[0].rows;
		if(!this.domNode.className.match("NoSmallTableMode") && this.domNode.offsetWidth > this.domNode.parentNode.offsetWidth + 2) { //2px tolerance for the table border
			setTimeout(dojo.hitch(this, function() {
				if(!this.domNode.className.match("ContentListForcedWidth")) {
					this._enableSmallTableMode(this.domNode.rows);
					dojo.addClass(this.domNode, "ContentListForcedWidth");
				}
				if(tRows[1]) {
					var firstContentCell = tRows[1].cells[0].className.match("Checkbox") ? tRows[1].cells[1] : tRows[1].cells[0];
					this._calcCellWidth(firstContentCell);
				}
			}), 1);
		}
	},

	postCreate: function() {

		this.inherited("postCreate", arguments);

		this.domNode.className += " RowSelection" + (epages.Browser.engine == "MSIE" ? "" : " HideElement");

		dojo.connect(window, "resize", this, "_setCheckTableMode");
		dojo.subscribe("RightSidebar/afterShowSidebar", dojo.hitch(this, function() {
			setTimeout(dojo.hitch(this, function() {
				this._setCheckTableMode();
			}), 1);
		}));
		this._setCheckTableMode();

		var tRows = this.domNode.tBodies[0].rows;

		for(var i = 0; i < tRows.length; i++) {

			var relevantInputCell = this._getRelevantInputCell(tRows[i]);

			if(relevantInputCell) {
				// the listener does not know where to find the relevant inputs unless we mark the parent cell
				relevantInputCell.className += " RowSelectionInputCell";
				dojo.connect(tRows[i], "click", function(evt) {
					var someThingSelected;
					if(document.selection && document.all) {
						someThingSelected = document.selection.createRange().text;
					}else {
						someThingSelected = window.getSelection.rangeCount;
					}
					var realTargetTag = evt.target.tagName.toLowerCase();
					// be sure that we don't override existing functionality
					if((evt.target.className.match("RowSelector") && !evt.target.className.match("NoRowSelector")) || (
							!someThingSelected &&
							realTargetTag != "a" &&
							realTargetTag != "input" &&
							realTargetTag != "select" &&
							realTargetTag != "option" &&
							realTargetTag != "textarea" &&
							realTargetTag != "img" &&
							realTargetTag != "label" &&
							!evt.target.className.match("NoRowSelector") &&
							!evt.target.className.match("Sprite"))
					) {
						var inputCell = null;
						if(realTargetTag == "td" && evt.target.getElementsByTagName("input").length > 0) {
							// the clicked element is a table cell and contains inputs
							// (although it is not our declarated cell)
							inputCell = evt.target;
						}else {
							// get the previous declarated cell (by class)
							var cells = this.getElementsByTagName("td");
							for(var i = 0; i < cells.length; i++) {
								if(cells[i].className.match("RowSelectionInputCell")) {
									inputCell = cells[i];
									break;
								}
							}
						}
						var inputs = inputCell.getElementsByTagName("input");
						var radioInputs = 0;
						// precheck how many radio inputs given (for handling yes/no inputs)
						for(var i = 0; i < inputs.length; i++) {
							if(inputs[i].className.match("Radio")) {
								radioInputs++;
							}
						}
						// now it's getting serious
						for(var i = 0; i < inputs.length; i++) {
							if(!inputs[i].disabled && inputs[i].type != "hidden") {
								if(inputs[i].className.match("Radio")) {
									if(radioInputs > 1 && inputs[i].checked == false) {
										// it is a double radio input (yes/no)
										// (therefore we only activate the unchecked input)
										epages.event.fire(inputs[i], "click");
										break;
									}else {
										// it is a single radio input (content list selector)
										epages.event.fire(inputs[i], "click");
									}
								}else if(inputs[i].className.match("Checkbox")) {
									// it is a checkbox
									epages.event.fire(inputs[i], "click");
									if(!inputs[i].rowSelectEventSet) {
										inputs[i].rowSelectEventSet = true;
										dojo.connect(inputs[i], "click", function(evt) {
											if(evt.currentTarget.checked && evt.target.parentNode.cellIndex == 0) {
												dojo.addClass(evt.currentTarget.parentNode.parentNode, "RowSelected");
											}else {
												dojo.removeClass(evt.currentTarget.parentNode.parentNode, "RowSelected");
											}
										});
									}
								}else {
									// it is something else
									try {
										inputs[i].focus();
										inputs[i].select();
									}catch(e){}
									break;
								}
							}
						}
					}else if(!evt.target.rowSelectEventSet && realTargetTag == "input" && evt.target.className && evt.target.className.match("Checkbox")) {
						// we have to handle checkboxes even if the row never has been clicked before
						if(evt.target.checked && evt.target.parentNode.cellIndex == 0) {
							dojo.addClass(evt.target.parentNode.parentNode, "RowSelected");
						}else {
							dojo.removeClass(evt.target.parentNode.parentNode, "RowSelected");
						}
					}
				});
			}
		}
		setTimeout(dojo.hitch(this, function() {
			dojo.removeClass(this.domNode, "HideElement");
		}), 1);
	}
});