/*
	Copyright (c) 2006-2010, ePages GmbH
	All Rights Reserved.

	epages.widget.SlideBanner $Revision: 1.1.2.1 $

	usage:
	<div dojoType="epages.widget.SlideBanner">
		<ul class="SlideContainer">
			<li>
				<a href="#"><img src="images/test/1.gif" /></a>
			</li>
			<li>
				<a href="#"><img src="images/test/2.gif" /></a>
			</li>
			<li>
				<a href="#"><img src="images/test/3.gif" /></a>
			</li>
		</ul>
	</div>

*/
dojo.provide("epages.widget.SlideBanner");
dojo.require("epages.widget");
dojo.require("dojo.fx");
	
dojo.declare("epages.widget.SlideBanner", [dijit._Widget], {
	containerWidth  : '400',				// string/integer - width of display container e.g. width="400"
	slideWidth      : '400',				// string/integer - width of slide container e.g. width="400"
	slideHeight     : '300',				// string/integer - height of slide container e.g. width="300"

	/*
	 * nodes
	 */
	slideContainerNode	: undefined,		// domNode - dom node of slide container
	prevButtonNode			: undefined,		// domnode - dom node of previous button
	nextButtonNode			: undefined,		// domnode - dom node of next button
	infoBoxNode					: undefined,		// domnode - dom node of info box

	/*
	 * private
	 */
	_countSlides				: 0,						// integer - number of slides
	_slideNodeList			: undefined,		// dojo.NodeList - contains all slide nodes
	_currentSlideIndex	: 0,						// integer - current slide count 

	/*
	 * widget life cycle
	 */
	
	postMixInProperties : function() {
		this.inherited("postMixInProperties", arguments);
		this.slideWidth = parseInt(this.slideWidth);
		this.slideHeight = parseInt(this.slideHeight);
	},

	postCreate: function() {
		this.inherited("postCreate", arguments);
		
		dojo.addClass(this.domNode, "SlideBanner");
		var resultList = dojo.query(".SlideContainer");
		if(resultList) {
			this.slideContainerNode = resultList[0];
		} else {
			console.warn('Element with CSS Class ".SlideContainer" not found in '+this.declaredClass);
		}

		var slideWidth = this.slideWidth;			// Width of slide
		var slideHeight = this.slideHeight;		// Height of slide

		dojo.style( this.domNode , {
			"height" : slideHeight+"px",
			"width"  : slideWidth+"px"
		});

		this.prevButtonNode = dojo.create("span", { // CREATE: Previous button
				innerHTML	: "&lt;",
				onClick		: "return false",
				'class'		: "SlideButton SlideBannerLeft"
			}, this.domNode, "last");

		this.nextButtonNode = dojo.create("span", { // CREATE: Next button
				innerHTML	: "&gt;",
				onClick		: "return false",			
				'class'		: "SlideButton SlideBannerRight"
			}, this.domNode, "last");
			
		this.infoBoxNode = dojo.create(// CREATE: Info-Box
			"div", {				
				id : "infoBox"
			}, this.domNode, "after" );

			this._slideNodeList = dojo.query(".SlideContainer > li", this.domNode); // pickup dom nodes
			this._slideNodeList.forEach(function(node, index) { // calculate and set position of each single node
				dojo.style(node, {
					left			:(parseInt(index) * parseInt(slideWidth) + "px"),
					width			: slideWidth + "px",
					height		: slideHeight + "px"
				});
			});
			this._countSlides = this._slideNodeList.length;

			// Slider event handler
			this._connectButtons();
	},
	
	destory : function() {
		this._disconnectButtons();
	},
	
	/*
	 * private functions 
	 */
	_connectButtons: function() {
	// summary: initialize button events

		this.prevButtonHandler = dojo.connect(this.prevButtonNode, "onclick", this, function() { 
			this.onSlideIt(-1);
		});
		
		this.nextButtonHandler = dojo.connect(this.nextButtonNode, "onclick", this, function() { 
			this.onSlideIt(1);
		});
	},
	
	_disconnectButtons: function() {
	// summary: remove button events
		dojo.disconnect(this.prevButtonHandler);
		dojo.disconnect(this.nextButtonHandler);
	},
	
	/*
	 * public functions 
	 */

	onSlideIt: function (/* integer */ delta) {
	// summary: event function: slide to next or previous slide
			this._disconnectButtons();
			this._currentSlideIndex += delta;
			
			this._currentSlideIndex = (this._currentSlideIndex >= this._countSlides)? 0 : this._currentSlideIndex; 
			this._currentSlideIndex = (this._currentSlideIndex < 0)? this._countSlides-1 : this._currentSlideIndex;

			var newLeftPos = this._currentSlideIndex * (this.slideWidth * -1);

			var _this = this;
			var slideArgs = {
					node 			: this.slideContainerNode,
					duration	: 500,
					top 			: 0,
					left 			: newLeftPos,
					unit 			: "px",
					onEnd: function() { _this._connectButtons(); }
			};
			dojo.fx.slideTo(slideArgs).play(); // Start slide animation
	}
});