use strict;
use Test::Simple tests => 6;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject);
use DE_EPAGES::Object::API::Language;

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $System = LoadRootObject();
    my $Shop = $System->child( 'TestShop' );
    ok( $Shop->id, "create shop" );

    my $SANDBOX = $System->folder('EbayEnvironments')->child('SANDBOX');
    ok( $SANDBOX->id, "get sandbox environment" );

    my $ShopEnvFolder = $Shop->folder('EbayShopEnvs');
    my $ShopEnv = $ShopEnvFolder->insertChild('EbayShopEnvironment', {'Alias' => 'TestShopEnv',
                                                                      'Name' => 'TestShopEnv',
                                                                      'EbayEnvironment' => $SANDBOX});
    ok( $ShopEnv->id, "create shop environment" );

    my $AuctionFolder = $Shop->folder('EbayAuctions');
    my $Auction = $AuctionFolder->insertChild('EbayAuction', {'Alias' => 'TestAuction',
                                                              'Name' => 'TestAuction',
                                                              'EbayShopEnvironment' => $ShopEnv});
    ok( $Auction->id, "create auction" );

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    my $System = LoadRootObject();
    if( $System->existsChild( 'TestShop' ) ) {
        $System->child( 'TestShop' )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Shop', { Alias => 'TestShop' } );

    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
