 /*
  Copyright (c) 2006-2007, epages Software GmbH
  All Rights Reserved.

  epages.cartridges.de_epages.externalcontent.bingmap $Revision: 1.4.2.2 $

*/

dojo.provide('epages.cartridges.de_epages.externalcontent.bingmap');
dojo.require('epages.cartridges.de_epages.design.mcecontent');

epages.cartridges.de_epages.externalcontent.BingMap = epages.Class();

dojo.extend(epages.cartridges.de_epages.externalcontent.BingMap,
  epages.cartridges.de_epages.design.MceContent.prototype, {
  contentClass    : 'epages.cartridges.de_epages.externalcontent.BingMap',
  title           : 'bingmap',
  type            : undefined,
  preview         : epages.vars.StoreRoot + '/BO/icons/navelement_bingmap.png',
  
  map             : undefined,
  domObject       : undefined,
  centerLat       : 51.165,
  centerLng       : 10.455278,
  zoom            : 5,
  markerLng       : 11.583616,
  markerLat       : 50.928705,
  markerDesc      : '',
  markerText      : '',
  width           : undefined,
  height          : undefined,
  navId           : null,
  controls        : true,
  mapFixed        : false,
  pushpin         : undefined,
  puspinUnadded   : false,
  
  _showControlsNode: undefined,

  changeControls: function(){
  	if(!this.map) return;
    if(this.controls) this.map.HideDashboard();
    else              this.map.ShowDashboard();
      
    this.controls= !this.controls;
  },

  resize: function (width, height) {
    if(this.domObject == null)
      this.domObject = dojo.byId("map"+this.navId);
    if(this.map){
      this.domObject.style.width = width+'px';
      this.domObject.style.height = height+'px';
      this.map.Resize();
    }
    this.width = width;
    this.height = height;
  },

  changeSize: function(domObject){
    var size = domObject.value.split(":");
    this.resize(size[0], size[1]);
  },
  
  setMapStyle: function (type) {
      if(type === undefined){
        type = VEMapStyle.Aerial;
      }
      this.map.SetMapStyle(type);
  },
  
  _blockMapMove : function(){
    return true;
  },
  
  setMapFix: function(changeCheckBox){
    if(this._showControlsNode === undefined){
      this._showControlsNode = dojo.query("input[name='checkShowControls']")[0];
    }
    if(!this.mapFixed){
      this.map.AttachEvent("onmousedown", this._blockMapMove);
      this.map.AttachEvent("onmousewheel",this._blockMapMove);
      if(changeCheckBox !== undefined && changeCheckBox){
        this._showControlsNode.checked = false;
        this._showControlsNode.disabled = true;
      }
    }else{
      this.map.DetachEvent("onmousedown",this._blockMapMove);
      this.map.DetachEvent("onmousewheel",this._blockMapMove);
      if(changeCheckBox !== undefined && changeCheckBox){
        this._showControlsNode.checked = true;
        this._showControlsNode.disabled = false;
      }
    }
    this.mapFixed = !this.mapFixed;
    
    if(this.controls==this.mapFixed)
    {
        this.changeControls();
    }
  },
  
  setMarkerText : function(domObject){
      var description = domObject.value;
      description = description.replace(/\n/g,'<br />');
      if(this.pushpin === undefined){
        this.pushpin = new VEShape(VEShapeType.Pushpin, this.map.GetCenter());  
        this.pushpinUnshown = true;
      }
      this.pushpin.SetTitle(description);
      this.markerText = description;
  },
  
  setMarkerPosition : function(point,address){
    if(point === undefined)
      point = this.map.GetCenter();

    if(this.pushpin === undefined){
      this.pushpin = new VEShape(VEShapeType.Pushpin, point);
      this.pushpin.SetDescription(address);
      this.pushpin.SetTitle("-");
      this.markerText = "-";
      this.map.AddShape(this.pushpin);  
    }
    else{
      if(this.pushpinUnshown)
        this.map.AddShape(this.pushpin);
      this.pushpin.SetPoints([point]);
      this.pushpin.SetDescription(address);
    }
    this.markerLat = point.Latitude;
    this.markerLng = point.Longitude;
    this.markerDesc = address;
  },
  
  findAddress: function (address) {
    this.map.Find("",
                  address,
                  null,
                  null,
                  0,
                  20,
                  false,
                  false,
                  false,
                  true,
                  dojo.hitch(this,function(a,b,c){if(c==null)alert("Address not found.");else{this.setMarkerPosition(c[0].LatLong,address);}}));
  },
  loadMap: function( domObject, info ) {
    this.domObject = domObject;
    console.debug(domObject.id);
    this.map = new VEMap(domObject.id);
    var loadIt = function (){
      if (info === undefined) {
        this.map.LoadMap();//new VELatLong(this.centerLat, this.centerLng), this.zoom,'h');
      } else {
        info = this.decodeInfo(info);
        if (info.controls.replace !== undefined)
          info.controls = info.controls == 'true' ? true : false;
        if (info.zoom.replace !== undefined)
          info.zoom = parseInt(info.zoom, 10);
        this.map.LoadMap(new VELatLong(info.centerLat, info.centerLng), info.zoom,info.type);
        if (info.controls == false) this.changeControls();
        if (info.mapFixed == 'true'){ this.mapFixed = false; this.setMapFix();}
        this.controls = info.controls;
        this.resize(info.width, info.height);
        if (info.markerText !== undefined && info.markerText != '') {
          this.markerText = info.markerText;
          this.markerDesc = info.markerDesc;
              
          this.pushpin = new VEShape(VEShapeType.Pushpin, new VELatLong(info.markerLat,info.markerLng));
          this.pushpin.SetTitle(this.markerDesc);
          this.pushpin.SetDescription(this.markerText);
          this.map.AddShape(this.pushpin);  
        }
      }
    };
    var interval = setInterval(dojo.hitch(this,function(){
    	if ((eval("typeof VEMap") != "undefined") && (domObject.attachEvent != undefined)){
    		clearInterval(interval);dojo.hitch(this,loadIt)();
    	}
    }), 10); 
  },

  info: function () {
    // save map detail at object
    var map = this.map;
    if (map !== undefined) {
      dojo.mixin(this, {
        centerLat : map.GetCenter().Latitude,
        centerLng : map.GetCenter().Longitude,
        type    : map.GetMapStyle(),
        zoom    : map.GetZoomLevel()
      });
    }
    // get info details from object
    var info = epages.cartridges.de_epages.design.MceContent.prototype.info.apply(this);
    dojo.mixin(info, {
      centerLat : this.centerLat,
      centerLng : this.centerLng,
      controls  : this.controls,
      type      : this.type,
      zoom      : this.zoom,
      width     : this.width,
      height    : this.height,
      markerLng : this.markerLng,
      markerLat : this.markerLat,
      markerDesc: this.markerDesc,
      markerText: this.markerText,
      mapFixed  : this.mapFixed
    });
    return info;
  },

  LOAD: function (domObject, info) {
    var init = function () {
      if (window.VEMap === undefined) {
        domObject.innerHTML = 'Preview';
      } else {
        var bingMap = new epages.cartridges.de_epages.externalcontent.BingMap();
        bingMap.loadMap(domObject, info);
      }
    };
      
    if (window.VEMap === undefined)
      window.setTimeout(init, 100);
    else
      dojo.hitch(this,init)();
  },

  contentString: function (info) {
    var timestamp = (new Date()).getTime();
    var aStrings = [
      '\n<script type="text/javascript" src="https://ecn.dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.2"></script>',
      '<script type="text/javascript">',
      'dojo.addOnLoad(function () {',
      'dojo["require"]("epages.cartridges.de_epages.externalcontent.bingmap");',
      'epages.cartridges.de_epages.externalcontent.BingMap.prototype.LOAD($(\'map'+ timestamp +'\'), \'' + this.encodeInfo(info) + '\');',
      '});',
      '</script>',
      '<div id=map'+ timestamp +' style="position:relative; width: '+ info.width +'px; height: '+info.height+'px">\n</div>'
    ];
    return aStrings.join("\n");
  },

  loadPreview: function (domObject){
  	console.log("loadpreview");
    var info = epages.vars.externalcontent[this.navId].object.info();
    info = this.encodeInfo(info);
    var outerDiv = document.createElement('div');
    outerDiv.innerHTML = '<div id="map'+this.navId+'" style="position:relative; width: 300px; height: 300px;">Loading Map...</div>';
    outerDiv.id ="PluginPreviewFrame";
    outerDiv.style.overflow ="auto";
    domObject.parentNode.replaceChild(outerDiv,domObject);
    window["epages_vars_externalcontent_" + this.navId + "_callback"] = dojo.hitch(this, function() { this.loadMap($('map'+this.navId),info); });
    var headNode = dojo.query("head");
    dojo.create("script",{
      "src" : "https://ecn.dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.2&onscriptload=epages_vars_externalcontent_"+this.navId+"_callback",
      "type": "text/javascript"
    }, headNode[0]);
  }
});

