use Test::More;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath Transaction);
use DE_EPAGES::Fianet::API::FianetRequest qw(DoFianetTracking);
use DE_EPAGES::Order::API::Order qw (CreateOrder);

my $ShopPath = "/Shops/DemoShop";
my $UserPath = "/Shops/DemoShop/Customers/1001";

sub testFianet {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            
            SKIP: {
                
                my $Shop;
                $Shop = LoadObjectByPath($ShopPath) if ExistsObjectByPath($ShopPath);
                plan skip_all => 'no demoshop' unless (defined $Shop);
                plan skip_all => 'demoshop no internal testshop' unless $Shop->get('IsInternalTestShop');
                
                my $Customer;
                $Customer = LoadObjectByPath($UserPath) if ExistsObjectByPath($UserPath);
                plan skip_all => 'no demoshop user' unless (defined $Customer);
                
                my $LocalOrder = CreateOrder($Customer, {'CurrencyID' => 'EUR' });
                $LocalOrder->get('LineItemContainer')->set({'GrandTotal'    => 1});
                
                # test tracking request
                ok(TestTrackingRequest($LocalOrder),'TRACKING');
                
                Transaction( sub {
                    $LocalOrder->delete;
                } );
            }
        });
}


sub TestTrackingRequest {
    my ( $Order) = @_;
    
    my $Servlet =undef;
    my $hTrackingParams = { 'REMOTE_ADDR' => '127.0.0.1' };
    return DoFianetTracking($Order,$hTrackingParams)->{'response_code'} eq 'OK';

}

sub Main {
    testFianet();
    done_testing();
    return;
}

RunScript( Sub => \&Main );

1;