use Test::More;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Installer::API::Object::PatchCodeGenerator;
use strict;

sub Main {
    my $CodeGen = DE_EPAGES::Installer::API::Object::PatchCodeGenerator->new();

    my $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_droppedindex.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->dropIndexIfExists('amazonoffer','i_productid');}, 'dropped index');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_droppedindex_twice.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->dropIndexIfExists('amazonoffer','i_productid');$dbi->dropIndexIfExists('amazonoffer','i_portalconfigid_status');}, 'dropped two indeces');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->createIndex('amazonoffer','i_amazonitemprice',undef,['amazonofferid'])unless($dbi->existsIndex('amazonoffer','i_amazonitemprice'));}, 'added index (default sort)');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_asc.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->createIndex('amazonoffer','i_amazonitemprice',undef,['amazonofferid'],{'amazonofferid'=>'ASC'})unless($dbi->existsIndex('amazonoffer','i_amazonitemprice'));}, 'added index (ASC sort)');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_desc.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->createIndex('amazonoffer','i_amazonitemprice',undef,['amazonofferid'],{'amazonofferid'=>'DESC'})unless($dbi->existsIndex('amazonoffer','i_amazonitemprice'));}, 'added index (DESC sort)');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_multicolumn.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->createIndex('amazonoffer','i_amazonitemprice',undef,['parentofferid','amazonofferid'],{'amazonofferid'=>'ASC'})unless($dbi->existsIndex('amazonoffer','i_amazonitemprice'));}, 'added index on two columns');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_changedindex.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->recreateIndex('amazonoffer','i_productid',undef,['productid'],{'productid'=>'DESC'});}, 'changed index sort order');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_unique.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->createIndex('amazonoffer','i_amazonitemprice','UNIQUE',['amazonofferid'],{'amazonofferid'=>'ASC'})unless($dbi->existsIndex('amazonoffer','i_amazonitemprice'));}, 'added unique index');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_unique_clustered.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->createIndex('amazonoffer','i_amazonitemprice','UNIQUECLUSTERED',['amazonofferid'],{'amazonofferid'=>'ASC'})unless($dbi->existsIndex('amazonoffer','i_amazonitemprice'));}, 'added unique clustered index');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_addedindex_nonclustered.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->createIndex('amazonoffer','i_amazonitemprice','NONCLUSTERED',['amazonofferid'],{'amazonofferid'=>'ASC'})unless($dbi->existsIndex('amazonoffer','i_amazonitemprice'));}, 'added nonclustered index');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_control_01_changedindex_twice.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{$dbi->recreateIndex('amazonoffer','i_productid',undef,['productid'],{'productid'=>'DESC'});$dbi->recreateIndex('amazonoffer','i_parentofferid',undef,['parentofferid']);}, 'changed two indeces: sort order and clustered to default');

    $Result = $CodeGen->getCodeForDiffInSQLFiles({
        'index_ref_01.sql' => 'index_ref_01.sql'
    },'Sybase');
    is(OnlyDbiStatements($Result),q{}, 'no changes');

}

sub OnlyDbiStatements {
    my $string = shift;
    my @matches = ($string =~ m/(\$dbi->.*;)/gm);
    $string = join "\n", @matches;
    $string =~ s/\s//g;
    return $string;
}

RunScript( Sub => \&Main );

done_testing();

1;
