#!perl -w
use strict;
use Test::Simple tests => 28;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::SQLCompare qw ( CompareSQL );


RunScript(Sub => sub {
    ok(CleanUp(), "CleanUp");

    #tables
    ok((not CompareSQL('ref(01).sql','control(01).sql')), "register too much table");
    ok((not CompareSQL('control(01).sql','ref(01).sql')), "register missing table");

    #columns
    ok((not CompareSQL('ref(02).sql', 'control(02).sql')), "register missing column");
    ok((not CompareSQL('ref(02).sql', 'control(02)(02).sql')), "register changed position for column");
    ok((not CompareSQL('ref(02).sql', 'control(02)(03).sql')), "register changed not null value for column 1st");
    ok((not CompareSQL('ref(02).sql', 'control(02)(04).sql')), "register changed not null value for column 2nd");
    ok((not CompareSQL('ref(02).sql', 'control(02)(05).sql')), "register changed data type for column");
    ok((not CompareSQL('ref(02).sql', 'control(02)(06).sql')), "register changed default for column ");
    ok((not CompareSQL('ref(02).sql', 'control(02)(07).sql')), "register too much columns");

    #primary key
    ok((not CompareSQL('ref(03).sql', 'control(03)(01).sql')), "register wrong name of primary key");
    ok((not CompareSQL('ref(03).sql', 'control(03)(02).sql')), "register wrong column in primary key");
    ok((not CompareSQL('ref(03).sql', 'control(03)(03).sql')), "register too much columns in primary key");
    ok((not CompareSQL('ref(03).sql', 'control(03)(04).sql')), "register missing columns in primary key");

    #indices
    ok((not CompareSQL('ref(04).sql', 'control(04).sql')), "register missing index");
    ok((not CompareSQL('ref(04).sql', 'control(04)(02).sql')), "register missing indices");
    ok((not CompareSQL('ref(04).sql', 'control(04)(03).sql')), "register diff in index column sort order 1st");
    ok((not CompareSQL('ref(04).sql', 'control(04)(04).sql')), "register diff in index column sort order 2nd");
    ok((not CompareSQL('ref(04).sql', 'control(04)(05).sql')), "register diff in index column sort order 3rd");
    ok((not CompareSQL('ref(04).sql', 'control(04)(06).sql')), "register missing index columns");
    ok((not CompareSQL('ref(04).sql', 'control(04)(07).sql')), "register too much index columns");

    #foreign keys
    ok((not CompareSQL('ref(05).sql', 'control(05).sql')), "register too much foreign key");
    ok((not CompareSQL('ref(05).sql', 'control(05)(02).sql')), "register missing foreign key");

    ok((not CompareSQL('ref(06).sql', 'control(06).sql')), "register diff in foreign key column pairs 1st");
    ok((not CompareSQL('ref(06).sql', 'control(06)(02).sql')), "register diff in foreign key column pairs 2nd");
    ok((not CompareSQL('ref(06).sql', 'control(06)(03).sql')), "register diff in foreign key column pairs 3rd");
    ok((not CompareSQL('ref(06).sql', 'control(06)(04).sql')), "register too much column pairs");
    ok((not CompareSQL('ref(06).sql', 'control(06)(05).sql')), "register missing column pairs");
}, Debug=>1);

sub CleanUp {
    return 1;
}
1;