use Test::Simple tests => 25;
use strict;

use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );

local $ENV{'EPAGES_TESTSITE'} = 'Site';

sub Main {
    ConnectTestSite();

    my $System = LoadRootObject();

    #remember old configuration
    my $ahCountriesOld      = $System->get('Countries');
    my $CountriesOldCount   = scalar @$ahCountriesOld;

    my $ahCurrenciesOld     = $System->get('Currencies');
    my $CurrenciesOldCount  = scalar @$ahCurrenciesOld;

    my $ahLanguagesOld      = $System->get('Languages');
    my $LanguagesOldCount   = scalar @$ahLanguagesOld;

    my $ahLocalesOld      = $System->get('Locales');
    my $LocalesOldCount   = scalar @$ahLocalesOld;


    #do the import of test standards
    ImportObjects( 'ImportStandards.xml' );


    #get the new configuration
    my $ahCountries = $System->get('Countries');
    my $CountriesCount = scalar @$ahCountries;
    my %TestCountries; @TestCountries{map{$_->{CountryID}}@$ahCountries} = @$ahCountries;

    my $ahCurrencies = $System->get('Currencies');
    my $CurrenciesCount = scalar @$ahCurrencies;
    my %TestCurrencies; @TestCurrencies{map{$_->{CurrencyID}}@$ahCurrencies} = @$ahCurrencies;

    my $ahLanguages = $System->get('Languages');
    my $LanguagesCount = scalar @$ahLanguages;
    my %TestLanguages; @TestLanguages{map{$_->{Code3}}@$ahLanguages} = @$ahLanguages;

    my $aoLocales = $System->get('Locales');
    my $LocalesCount = scalar @$aoLocales;
    my %TestLocales; @TestLocales{map{$_->get('LocaleID')}@$aoLocales} = @$aoLocales;


    #do the tests
    ok( ($CountriesCount-$CountriesOldCount)==3, 'Countries import count' );
    ok( defined $TestCountries{999}, 'Country 999 imported' );
    ok( $TestCountries{999}->{Native} eq 'Test Country', 'Country 999 native' );
    ok( defined $TestCountries{998}, 'Country 998 imported' );
    ok( $TestCountries{998}->{Native} eq 'Test Country 2', 'Country 999 native' );
    ok( defined $TestCountries{997}, 'Country 997 imported' );
    ok( $TestCountries{997}->{Native} eq 'Test Country 3', 'Country 999 native' );

    ok( ($CurrenciesCount-$CurrenciesOldCount)==1, 'Currencies import count' );
    ok( defined $TestCurrencies{YYY}, 'Currency YYY imported' );
    ok( $TestCurrencies{YYY}->{Name} eq 'Test Currency', 'Currency YYY name' );

    ok( ($LanguagesCount-$LanguagesOldCount)==1, 'Languages import count' );
    ok( defined $TestLanguages{YYY}, 'Language YYY imported' );
    ok( $TestLanguages{YYY}->{English} eq 'Test Language', 'Language YYY name' );

    ok( ($LocalesCount-$LocalesOldCount)==1, 'Locales import count' );
    ok( defined $TestLocales{YY_YX}, 'Locale YY_YX imported' );


    #try to rebuild the old configuration
    DeleteObjects( 'DeleteStandards.xml' );

    #get the restored configuration
    $ahCountries = $System->get('Countries');
    my $CountriesCountNew = scalar @$ahCountries;
    my %TestCountriesNew; @TestCountriesNew{map{$_->{CountryID}}@$ahCountries} = @$ahCountries;

    $ahCurrencies = $System->get('Currencies');
    my $CurrenciesCountNew = scalar @$ahCurrencies;
    my %TestCurrenciesNew; @TestCurrenciesNew{map{$_->{CurrencyID}}@$ahCurrencies} = @$ahCurrencies;

    $ahLanguages = $System->get('Languages');
    my $LanguagesCountNew = scalar @$ahLanguages;
    my %TestLanguagesNew; @TestLanguagesNew{map{$_->{Code3}}@$ahLanguages} = @$ahLanguages;

    $aoLocales = $System->get('Locales');
    my $LocalesCountNew = scalar @$aoLocales;
    my %TestLocalesNew; @TestLocalesNew{map{$_->get('LocaleID')}@$aoLocales} = @$aoLocales;

    #check if the configuration restored
    ok( $CountriesCountNew == $CountriesOldCount, 'restored Countries import count' );
    ok( !defined $TestCountriesNew{999}, 'Country 999 deleted' );
    ok( !defined $TestCountriesNew{998}, 'Country 998 deleted' );
    ok( !defined $TestCountriesNew{997}, 'Country 997 deleted' );

    ok( $CurrenciesCountNew == $CurrenciesOldCount, 'restored Currencies import count' );
    ok( !defined $TestCurrenciesNew{YYY}, 'Currency YYY deleted' );

    ok( $LanguagesCountNew == $LanguagesOldCount, 'restored Languages import count' );
    ok( !defined $TestLanguagesNew{YYY}, 'Language YYY deleted' );

    ok( $LocalesCountNew == $LocalesOldCount, 'restored Locales import count' );
    ok( !defined $TestLocalesNew{YY_YX}, 'Locale YY_YX deleted' );

    DisconnectSite();
}


RunScript(Sub=>\&Main, Debug=>1);
