#!perl -w
use strict;
use Test::Simple tests => 11;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::XMLCompare qw ( CompareXML );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Object::API::XMLStructureExportDriver;

RunScript(Sub => sub {
    ok(CleanUp(), "CleanUp");

    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $ExportDriver = DE_EPAGES::Object::API::XMLStructureExportDriver->new();
    my $Tmp = 'Temp.xml';
    my $TransTmp = 'Translation.Tmp.tt.xml';

    my $XML = 'AttributesObject.xml';
    my $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    $XML = 'AttributesSystem.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    $XML = 'Dependencies.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    $XML = 'HooksObject.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    $XML = 'HooksPreDefAttribute.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    #$XML = 'Languages.xml';
    #$XMLStruct = $ImportHandler->getXMLStructure($XML);
    #$ExportDriver->exportXML($Tmp, $XMLStruct);
    #ok(
    #    (
    #        CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
    #        CompareXML($Tmp, $XML, {'printmessage' => '1'})
    #    )
    #    , "$XML export and original xml are equal"
    #);

    $XML = 'Locales.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    $XML = 'System.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    $XML = 'Translation_Attributes_de.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($TransTmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $TransTmp, {'printmessage' => '1'}) and
            CompareXML($TransTmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    $XML = 'UnitsOfMeasurement.xml';
    $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($Tmp, $XMLStruct);
    ok(
        (
            CompareXML($XML, $Tmp, {'printmessage' => '1'}) and
            CompareXML($Tmp, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    ok(
        not(
            CompareXML('System.xml', $Tmp,) and
            CompareXML($Tmp, 'System.xml',)
        )
        , "export and control xml are not equal"
    );

}, Debug=>1);

sub CleanUp {
    return 1;
}
1;