/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.order.sendorderdocument $Revision: 1.2.2.4 $

*/

dojo.provide("epages.cartridges.de_epages.order.sendorderdocument");
dojo.require("epages.widget.FormElement");
dojo.require("epages.widget.Modaldialog");
dojo.require("epages.uimessagehandler");
dojo.require("epages.html.element");
dojo.require("epages.string");

epages.cartridges.de_epages.order.SendOrderDocument = {

	errorMap: {
		EMAIL_MISSING:															"",
		InvalidMailAddress:													"",
		SUBJECT_MISSING:														"",
		MISSING_FROM:																"",
		No_Recipients:															"",
		Invalid_Recipient:													"",
		ERROR_XML_TAG_ENDING_MISMATCH:							"",
		ERROR_ATTRIBUTES_VALUE_OPEN_QUOTE_EXPECTED:	"",
		ERROR_XML_TAG_NOT_FINISHED:									"",
		ERROR_XML_DOCUMENT_ENDING_MISMATCH:					"",
		ERROR_CREATING_PDF:													""
	},

	localizedStrings: {
		sendDocumentAsPDF:													"",
		waitWhilePDFGenerating:											"",
		documentSentHeadline:												"",
		documentSentText:														"",
		sendMail:																		"",
		cancel:																			"",
		error:																			""
	},

	sendDocumentAsMailDialog: null,
	_translateStrings: function(dialogContent) {
        // summary:
	//	Retrieve all the errorMap / localizedStrings properties.
		if(this.errorMap.EMAIL_MISSING.length < 2) { // do this only once
			var tmpContainer = document.createElement("div");
			tmpContainer.className = "HideElement";
			tmpContainer.innerHTML = dialogContent;
			eval(tmpContainer.getElementsByTagName("script")[0].innerHTML);
		}
	},

	_showBusyLayer: function() {
		dojo.publish("uimessage/startBusy", ["sendorderdocument_busy", this.localizedStrings.waitWhilePDFGenerating]);
	},

	_hideBusyLayer: function() {
		dojo.publish("uimessage/stopBusy", ["sendorderdocument_busy"]);
	},
	_insertDialogMessage: function(isError, messageHeadline, messageText) {
        // summary:
	//		Pass the status message of this module to the user via .DialogMessage.
	// description:
	//		Can be the error or success message.
		// this is tricky because dialog areas don't have a fixed id
		// we're using the <form> (which has to be there in every case) as fallback
		var dialogNode = dojo.query("div.TabPage > form > div.Dialog")[0] || dojo.query("div.TabPage > form")[0];

		var messageNode = $("SendOrderDocumentResult") || document.createElement("div");
		messageNode.id = "SendOrderDocumentResult"
		messageNode.className = "DialogMessage " + (isError ? "MessageWarning" : "MessagePositiveResult");
		messageNode.innerHTML = "<h3>" + messageHeadline + "</h3><p>" + messageText + "</p>";

		dialogNode.insertBefore(messageNode, dialogNode.firstChild);
	},

	_throwError: function(errorMessage) {
		this._insertDialogMessage(true, this.localizedStrings.error, errorMessage || this.errorMap.ERROR_CREATING_PDF);
		this._hideBusyLayer();
	},

	_onMailSent: function() {
		this._insertDialogMessage(false, this.localizedStrings.documentSentHeadline, this.localizedStrings.documentSentText);
		this._hideBusyLayer();
	},

	_onResponse: function(response) {
		if(!response || response.error) {
			// hard error with no further information
			this._throwError(this.errorMap.ERROR_CREATING_PDF);
		}else if(response.data.mailSent) {
			if(epages.string.toBoolean(response.data.mailSent)) {
				// everything fine
				this._onMailSent();
			}else {
				// soft error with description
				this._throwError(this.errorMap[response.data.reason]);
			}
		}
	},

	_removeErrorMessageFromInput: function(evt) {
			dojo.removeClass(evt.currentTarget, "DialogError");
			dojo.publish("uimessage/hide");
	},

	_onSendButtonClicked: function(objectID) {
        // summary:
	//		Trigger the xhr process or init error handling.
		var sendOrderDocumentForm = $("sendOrderDocumentForm");
		var inputPosition = $E(sendOrderDocumentForm.To).getAbsolutePosition(dojo.body());

		if(!sendOrderDocumentForm.To.value) {
			dojo.publish("uimessage/show", [this.localizedStrings.error, this.errorMap.EMAIL_MISSING, "Bubble", {
				x:					inputPosition.x,
				y:					inputPosition.y + sendOrderDocumentForm.To.offsetHeight - 5,
				sizeClass:	"Small"
			}]);
			dojo.addClass(sendOrderDocumentForm.To, "DialogError");
			dojo.connect(sendOrderDocumentForm.To, "focus", this, "_removeErrorMessageFromInput");
			return;
		}else if(!sendOrderDocumentForm.To.value.match("@") || sendOrderDocumentForm.To.value.length < 3) {
			dojo.publish("uimessage/show", [this.localizedStrings.error, this.errorMap.InvalidMailAddress, "Bubble", {
				x:					inputPosition.x,
				y:					inputPosition.y + sendOrderDocumentForm.To.offsetHeight - 5,
				sizeClass:	"Small"
			}]);
			dojo.addClass(sendOrderDocumentForm.To, "DialogError");
			dojo.connect(sendOrderDocumentForm.To, "focus", this, "_removeErrorMessageFromInput");
			return;
		}
		if(!sendOrderDocumentForm.Subject.value) {
			inputPosition = $E(sendOrderDocumentForm.Subject).getAbsolutePosition(dojo.body());

			dojo.publish("uimessage/show", [this.localizedStrings.error, this.errorMap.SUBJECT_MISSING, "Bubble", {
				x:					inputPosition.x,
				y:					inputPosition.y + sendOrderDocumentForm.Subject.offsetHeight - 5,
				sizeClass:	"Small"
			}]);
			dojo.addClass(sendOrderDocumentForm.Subject, "DialogError");
			dojo.connect(sendOrderDocumentForm.Subject, "focus", this, "_removeErrorMessageFromInput");
			return;
		}

		this._showBusyLayer();


		var xhrUrlElements = ["?ObjectID=" + (objectID || "") + "&ViewAction=JSONViewResponse"];
		dojo.forEach(dojo.query("[name]", sendOrderDocumentForm), function(item) {
			if(item.value) {
				// collect all form values and append them to the xhr url
				item.value = item.value.replace(/\n|\r/g, "%0A");
				xhrUrlElements.push("&" + item.name + "=" + item.value);
			}
		});
		(new epages.io.Json()).loadAsync(xhrUrlElements.join(""), dojo.hitch(this, function(response) {this._onResponse(response)}));
		this.sendDocumentAsMailDialog.hide();
	},

	openDialog: function(docAlias /* String */, objectID /* String */, languageID /* String */) {
        // summary:
        //		Open the dialog for sending a mail with pdf attached.
        // description:
        //		The opened dialog triggers the method which will generate the pdf equivalent to the current order document.
        // docAlias:
        //		The alias (TLE) for the document.
        // objectID:
        //		The ObjectID (TLE) for the document.
        // languageID:
        //		The LanguageID (TLE) for the document.
		this.sendDocumentAsMailDialog = $$("sendDocumentAsMailDialog");
		if(this.sendDocumentAsMailDialog) {this.sendDocumentAsMailDialog.destroy()} // the dialog was opened before
		var dialogContent = (new epages.io.Json()).loadSync("?", { // maybe async would do? Sync frezes the UI.
			ViewAction:         "JSONSnippet",
			TemplateAction:     "MBO-ViewSendOrderDocumentAsPDFMailDialog",
			TemplateLanguageID: (languageID || ""),
			ObjectID:           (objectID || ""),
			Doc:                (docAlias || "")
		}).data.snippet;
		this._translateStrings(dialogContent);
		this.sendDocumentAsMailDialog = new epages.widget.Modaldialog({
			id: "sendDocumentAsMailDialog",
			options: {
				title : this.localizedStrings.sendDocumentAsPDF,
				width : 450,
				scroll: "hidden",
				content: dialogContent
			},
			buttons: [{
				label: this.localizedStrings.sendMail,
				onclick: dojo.hitch(this, function(){this._onSendButtonClicked(objectID)})
			},{
				label: this.localizedStrings.cancel,
				onclick: dojo.hitch(this, function() {
					this.sendDocumentAsMailDialog.hide();
				})
			}]
		});

		dojo.connect(this.sendDocumentAsMailDialog.domNode, "mouseup", function() {
			// hide all error messages
			dojo.publish("uimessage/hide");
		});

		dojo.connect(this.sendDocumentAsMailDialog.domNode, "keypress", dojo.hitch(this, function(evt) {
			if(evt.keyCode === dojo.keys.ENTER && evt.target.tagName !== "TEXTAREA") {
				// enable the user to submit the form via keyboard
				this._onSendButtonClicked(objectID);
			}
		}));

		dojo.body().appendChild(this.sendDocumentAsMailDialog.domNode);
		this.sendDocumentAsMailDialog.show();

		// for usability enhancements we select the content of the first input element
		var firstInput = dojo.query("input[type=text]", this.sendDocumentAsMailDialog.domNode)[0];
		setTimeout(dojo.hitch(this, function() {
			epages.event.fire(firstInput, "mouseup");
			try {
				firstInput.focus();
				firstInput.select();
			}catch(e) {}
		}), this.sendDocumentAsMailDialog.duration * 1.2); // the animation resets the focus, so we have to wait until it finished
	}
};