IF OBJECT_ID('i_Session') IS NOT NULL
    DROP PROCEDURE i_Session
GO
CREATE PROCEDURE i_Session (
    @IntSessionID         int          = null
,   @SessionID            varchar(32)
,   @UserID               int          = null
,   @IsClosed             bit
,   @IsCookieIdentified   bit
,   @CreationDate         datetime     = null
,   @RemoteIP             varchar(45)  = null
,   @RemoteHost           varchar(80)  = null
,   @RemoteUser           varchar(80)  = null
,   @RefererURL           varchar(255) = null
,   @Token                varchar(32)  = null
) AS BEGIN
    DECLARE @TransactionCount int,
            @RowCounter int
    SELECT @TransactionCount = @@trancount

    IF @TransactionCount > 0 SAVE  TRANSACTION ti_session
    ELSE                     BEGIN TRANSACTION ti_session

    	while (@IntSessionID IS NULL) begin
                update session_nextid set id = id + 1, @IntSessionID = id + 1
                select @RowCounter = @@rowcount
                if (@RowCounter = 0) BEGIN
                    insert session_nextid (id) values (1)
                    select @IntSessionID = 1
                end
                if (@RowCounter > 1) BEGIN
                    delete session_nextid
                    insert session_nextid (id) values (1)
                    select @IntSessionID = 1
                end
                if exists ( select intsessionid from session where intsessionid = @IntSessionID)
                    select @IntSessionID = null
    	end
        IF @CreationDate IS NULL SELECT @CreationDate = getdate()

        INSERT INTO session
               ( intsessionid, sessionid, userid, isclosed, iscookieidentified, creationdate, remoteip, remotehost, remoteuser, refererurl, token)
        SELECT  @IntSessionID,@SessionID,@UserID,@IsClosed,@IsCookieIdentified,@CreationDate,@RemoteIP,@RemoteHost,@RemoteUser,@RefererURL,@Token

        IF NOT @@transtate IN (0, 1)
        BEGIN
            ROLLBACK TRANSACTION ti_session
            RETURN 1
        END

        SELECT @IntSessionID

    IF @TransactionCount = 0
        COMMIT TRANSACTION ti_session
    RETURN 0
END
GO
