use Test::More;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw( DatabaseConfig );
use DE_EPAGES::WebInterface::API::FakeRequest;

plan tests => 5;

sub Main {
    my $StoreName = 'REDIRECTTEST';
    my $Config = DatabaseConfig($StoreName);
    $Config->section( 'Connection', { 'redirect' => 'WebAdapterError.html' } );

    my $ServerName = 'localhost';
    my $URL = "http://$ServerName/epages/$StoreName.sf";

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new();
    my $Response = $FakeRequest->get( $URL );

    cmp_ok( $Response->code, '==', 503, '503 Service unavailable' );
    cmp_ok( $Response->header('Retry-After'), '==', 900, 'Retry-After: 900' );
    like( $Response->content, qr/Heavy Traffic/, 'content' );

    # test for non-existing database
    $StoreName = 'UNKNOWN';
    $URL = "http://$ServerName/epages/$StoreName.sf";
    $Response = $FakeRequest->get( $URL );

    cmp_ok( $Response->code, '==', 404, '404 Not Found' );
    like( $Response->content, qr/MissingConfigSection/, 'content' );
}

RunScript(Sub=>\&Main, Debug=>1);
