use Test::More tests => 3;
use warnings;
use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );
use DE_EPAGES::Presentation::API::Session qw( InsertSession );
use DE_EPAGES::Presentation::API::Table::Session qw( GetInfoSessionBySessionID DeleteSession );

sub Main {

    # fake
    local *DE_EPAGES::Presentation::API::Session::GetAppServer = sub {
        return Mock::AppServer->new;
    };

    # insert a session
    my $User = LoadObjectByPath('/Users/wsadmin');
    my $SessionID = InsertSession( $User, 1 );
    is( length $SessionID, 32, "SessionID length" );

    my $hSession = GetInfoSessionBySessionID( $SessionID );
    is( length $hSession->{'Token'}, 10, "Token length" );

    my $IntSessionID = $hSession->{'IntSessionID'};
    ok( $IntSessionID =~ /^[1-9]\d*$/, "IntSessionID is integer" );

    DeleteSession($IntSessionID);
}

RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

package Mock::AppServer;

sub new {
    my $class = shift;
    return bless {}, $class;
}

sub address {
    return '0.0.0.0';
}

sub port {
    return 0;
}

sub serverVariables {
    return {
        HTTP_REFERER => 'x',
        REMOTE_ADDR => 'ip',
        REMOTE_HOST => 'host',
        REMOTE_USER => 'user',
    };
}
