/*
	Copyright (c) 2006-2010, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.product.widget.ProductTypeAttributeList $Revision: 1.8.2.4 $

*/
dojo.provide("epages.cartridges.de_epages.product.widget.ProductTypeAttributeList");
dojo.require("epages.widget.LocalizedWidget");
dojo.require('epages.cartridges.de_epages.product.widget.ProductTypeAttribute');

dojo.declare("epages.cartridges.de_epages.product.widget.ProductTypeAttributeList",
	[epages.widget.LocalizedWidget], {

		/**
		 * public properties
		 */
		productTypeId     : '',														// string - id of product type
		productTypeName   : '',														// string - name of product type
		productTypeData   : undefined,										// object - product type data
		url               : '?',													//
		changeAction      : 'SaveProductType',						// string - name of change action
		viewAction        : 'View-JSON-ProductTypeData',	// string - name of view action
		objectId          : '',														// string - id of ressource
		currentLanguageId : '',					// string - id of current (display) language
		showVisibileOnly  : '0',				// string - flag to hide not visible attributes
		maxCountAttributes: 0,					// integer - numer of max possible attributes


		/**
		 * public properties
		 */
		_ep_childWidgets  : undefined,	// array   - array with child widgets
		_countAttributes  : 0,					// integer - current count attributes


		/**
		 * widget properties
		 */
		templatePath:    dojo.moduleUrl('epages.cartridges.de_epages.product.widget', "templates/ProductTypeAttributeList.html"),
		translationName: dojo.moduleUrl('epages.cartridges.de_epages.product.widget', 'templates/translation'),

		postMixInProperties: function(){
			this.inherited("postMixInProperties", arguments);
			this._ep_childWidgets = [];
			this.productTypeData = window.epages.vars.productTypeData;
			this.productTypeName = this.productTypeData.name;
			this.showVisibileOnly = epages.string.toBoolean(this.showVisibileOnly);

			if(epages.vars.features) {
				if(! epages.vars.features.ProductTypeAttributes) {
					console.warn("epages.vars.features.ProductTypeAttributes not defined in "+this.declaredClass );
				}
			} else {
				console.warn("epages.vars.features not defined in "+this.declaredClass );
			}
			this.maxCountAttributes = parseInt(epages.vars.features.ProductTypeAttributes.FeatureMaxValue);
		},

		postCreate: function(){
			this.inherited("postCreate", arguments);
			if(window.epages.vars.productTypeData) {
				console.warn('epages.vars.productTypeData not defined in '+this.declaredClass);
				this.productTypeData = window.epages.vars.productTypeData;
				this.productTypeName = this.productTypeData.name;
				this._setUpContent();
			}	else {
				this.onChangeProductType(this.productTypeId);
			}

			dojo.subscribe(this.id+"/attributeRemoved", this, "onAttributeRemoved");
			dojo.subscribe(this.id+"/changeProductType", this, "onChangeProductType");
			dojo.subscribe(this.id+'/changeProductTypeName', this, 'onChangeProductTypeName');
		},

		_setUpContent: function() {
		// summary: create attribute widgets / display feature count message
			this.nameNode.innerHTML = (this.productTypeData.name) ? this.productTypeData.name : this.productTypeData.alias;
			for(var key in this.productTypeData.attributes) {
				var data = this.productTypeData.attributes[key];
				if(data.isVisible == "1" || !this.showVisibileOnly) {
					var attributeWidget = new epages.cartridges.de_epages.product.widget.ProductTypeAttribute({
						name : data['name:'+this.currentLanguageId],
						alias : key,
						data: data,
						attributeListId : this.id,
						productObjectID : this.objectId,
						attributeNumber : this._countAttributes
					});
					if(attributeWidget) {
						this.containerNode.appendChild(attributeWidget.domNode);
						this._ep_childWidgets.push(attributeWidget);
					}
				}
				this._countAttributes++;
				if(this._countAttributes >= this.maxCountAttributes ) {
					this.newAttributeNode.style.display="none";
					this.featureLimitExeededNode.style.display = "";
				} else {
					this.newAttributeNode.style.display="";
					this.featureLimitExeededNode.style.display = "none";
				}
			}
		},

		onChangeProductTypeName: function(/* string */name ) {
		// summary: change the name node of productype
			this.nameNode.innerHTML = name;
		},

		onAddNewAttribute: function() {
		// summary: add ne (empty) attribute widget
			var attributeWidget = new epages.cartridges.de_epages.product.widget.ProductTypeAttribute({
				attributeListId : this.id,
				attributeNumber : this._countAttributes,
				productObjectID : this.objectId
			});
			if(attributeWidget) {
				this._ep_childWidgets.push(attributeWidget);
				this.containerNode.appendChild(attributeWidget.domNode);
				this._countAttributes++;
				if(this._countAttributes >= this.maxCountAttributes ) {
					this.newAttributeNode.style.display="none";
					this.featureLimitExeededNode.style.display = "";
				} else {
					this.newAttributeNode.style.display="";
					this.featureLimitExeededNode.style.display = "none";
				}
			}
		},

		onAttributeRemoved: function(/* object */ opt) {
		// summary: message function - decrease counter ( show / hide feature limit message)
			this._countAttributes--;
			if(this._countAttributes >= this.maxCountAttributes ) {
				this.newAttributeNode.style.display="none";
				this.featureLimitExeededNode.style.display = "";
			} else {
				this.newAttributeNode.style.display="";
				this.featureLimitExeededNode.style.display = "none";
			}
		},

		onChangeProductType : function(/* string*/ productTypeId) {
		// summary: load new product type data by product type id
			var formData = {
				'ChangeAction' : this.changeAction,
				'ViewAction' : this.viewAction,
				'ObjectID' : this.objectId,
				'Class' : productTypeId
			};

			var _this = this;
			var request = new dojo.xhrPost({
				url     : this.url,
				Accept  : 'text/x-json',
				content : formData,
				load    : function(response, args) {
					_this._setNewProductType(response, args);
				},
				error   : this._loadError
			});
		},

		_loadError: function (response, args) {
		// summary: error callback / displays an error message
			var rm = response.message;
			var rn = response.name;
			dojo.publish("uimessage/showErrors", ["IO ERROR", 'loading product type data', [{
				Message : 'Could not load product type data.('+rm+')',
				Reason  : rn
			}
			]]);
			console.error(response, args);
		},

		_setNewProductType : function(response, args) {
		// summary: prepare and setup new prdouctype data / widgets
			this._destroyChildWidgets();
			response = response.replace(/<!--.*-->/g, "");		// remove comments from pagetype
			this.productTypeData = dojo.fromJson(response);
			this._setUpContent();
		},

		_destroyChildWidgets : function() {
		// summary: destroys all childwidgets
			for(var el in this._ep_childWidgets) {
				this._ep_childWidgets[el].destroy();
				delete this._ep_childWidgets[el];
			}
			this._ep_childWidgets = [];
		}

});
