use strict;
use Test::More tests => 8;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store';

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $Product1 = LoadObjectByPath('/Test/TestShop/Products/MyPetrol');
    my $Product2 = LoadObjectByPath('/Test/TestShop/Products/MyRedNose');
    my $Product3 = LoadObjectByPath('/Test/TestShop/Products/MyYellowNose');

    my $ProductFolder = $Product1->parent;

    my $BundleProduct = $ProductFolder->insertChild('Product', {'Alias'=>'Set', 'IsBundleProduct' => 1});
    is( LoadObjectByPath('/Test/TestShop/Products/Set')->id, $BundleProduct->id, "Created bunded" );

    is ($Product1->get('StockLevel'), 5, 'StockLevel is imported correctly');
    $Product1->set({'StockLevel' => 4});
    is ($Product1->get('StockLevel'), 4, 'StockLevel is imported correctly');
    $Product1->set({'StockLevel' => 5});

    $Product1->addToBundle($BundleProduct, {'BundledQuantity' => 3});
    $Product2->addToBundle($BundleProduct, {'BundledQuantity' => 1});
    $Product3->addToBundle($BundleProduct, {'BundledQuantity' => 2});

    is ($BundleProduct->get('StockLevel'), 1, 'StockLevel is correct for bundleproduct');
    $BundleProduct->set({'StockLevel' => 3});
    is ($BundleProduct->get('StockLevel'), 1, 'Cannot set stocklevel at bundleproduct');

    $Product1->set({'StockLevel' => 6});
    $Product2->set({'StockLevel' => 3});
    $Product3->set({'StockLevel' => 4});

    is ($BundleProduct->get('StockLevel'), 2, 'StockLevel has changed after changing bundled products');

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
