﻿use strict;
use Test::Simple tests => 6;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::ProductRating::API::Object::ProductRating;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main {
    ConnectTestSite();
    ok( Init(), "initialization" );


    my $Shop = LoadObjectByPath('/Test/TestShop');


    my $Product = $Shop->folder('Products')->child('ht_0801099030');
    my $RatingFolder = $Shop->folder('Ratings');

    my $Class = LoadClassByAlias('ProductRating');

    my $nextAlias = $RatingFolder->nextAlias;

    my $ProductRating;

    for(my $i = 1; $i <= 10; $i++) {
        $ProductRating = $Class->insertObject({
            'Shop' =>   $Shop,
            'Parent' => $RatingFolder,
            'Alias' =>  $nextAlias.$i,
            'Product' => $Product,
            'Rating' => 3,
            'IsVisible' => 1,
            'Name'  => 'Marcel Reich-Ranicki',
        });

        $ProductRating->set({'Title'=> $i.' absolutes Scheissbuch'});
        $ProductRating->set({'Comment'=>'Hier hat nicht nur der Autor total versagt!'});
        $ProductRating->set({'EMail'=>'marcel@buch.de'});
    }

    ok(($Product->get('RatingAverage') == 3), "RatingAverage == 3");

    for(my $i = 1; $i <= 10; $i++) {
            $ProductRating = $Class->insertObject({
                        'Shop' =>   $Shop,
                        'Parent' => $RatingFolder,
                        'Alias' =>  $RatingFolder->nextAlias,
                        'Product' => $Product,
                        'Rating' => 1,
                        'IsVisible' => 1,
                        'Name'  => '1 Paulchen Panther',
                    });
            $ProductRating->set({'Title'=> $i.'Warum immer nur Tauben zum fressen??'});
            $ProductRating->set({'Comment'=> 'ich hatte eigentlich einen saftigen Affen erwartet!!'});
            $ProductRating->set({'EMail'=>'panther@tierpark.de'});
    }

    ok(($Product->get('RatingAverage') == 2), "RatingAverage == 2");

    $Product = $Shop->folder('Products')->child('cg_0100504001');
    for(my $i = 1; $i <= 10; $i++) {
            $ProductRating = $Class->insertObject({
                        'Shop' =>   $Shop,
                        'Parent' => $RatingFolder,
                        'Alias' =>  $RatingFolder->nextAlias,
                        'Product' => $Product,
                        'Rating' => 2,
                        'IsVisible' => 1,
                        'Name'  => 'Unwissender',
                    });
            $ProductRating->set({'Title'=> 'schönes Buch'});
            $ProductRating->set({'Comment'=> 'gut zu lesen, viele Bilder...'});
            $ProductRating->set({'EMail'=>'nobody@epages.de'});
    }

    ok(($Product->get('RatingAverage') == 2), "RatingAverage == 2");

    $ProductRating = $Class->insertObject({
        'Shop' =>   $Shop,
        'Parent' => $RatingFolder,
        'Alias' =>  $RatingFolder->nextAlias,
        'Product' => $Product,
        'Rating' => 5,
        'IsVisible' => 1,
        'Name'  => 'Unwissender',
    });
    $ProductRating->set({'Title'=> 'schönes Buch'});
    $ProductRating->set({'Comment'=> 'gut zu lesen, viele Bilder...'});
    $ProductRating->set({'EMail'=>'nobody@epages.de'});


    my $RatingAverage = $Product->get('RatingAverage');

    ok(( $RatingAverage eq "2.27272727272727"), "RatingAverage == 2,27272727272727");

    ok( Cleanup(), "cleanup" );
    DisconnectSite();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
