use Test::Simple tests => 7;
use strict;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw (ConnectTestSite DisconnectSite);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadObjectByPath);
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Presentation::Servlet::PageType;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);

local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};
my $ROOTPATH = '/';

RunScript( 'Sub'=> sub {
    ConnectTestSite();

    ok(Init(), "initialize");

    my $Object = LoadObjectByPath('/');
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'ObjectID' => $Object->id,
        'SearchDefinition' => GetSearchDefinition(LoadClassByAlias('Object'), 'LocalizedText'),
        'Parameters' => {
            'String'   => '%all%',
            'LanguageID' => GetPKeyLanguageByCode('de'),
            'Class.Alias' => 'TestObject',
        },
        'SortParameters'=> [
            'Class.Alias',
            'Alias'
        ],
    });
    $Search->statistic;
    ok ($Search->countItems == 3, 'found correct count');
    $Search->delete;

    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'ObjectID' => $Object->id,
        'SearchDefinition' => GetSearchDefinition(LoadClassByAlias('Object'), 'LocalizedText'),
        'Parameters' => {
            'String'   => '%ALL%',
            'LanguageID' => GetPKeyLanguageByCode('de'),
            'Class.Alias' => 'TestObject',
        },
        'SortParameters'=> [
            'Class.Alias',
            'Alias'
        ],
    });
    $Search->statistic;
    ok ($Search->countItems == 3, 'found correct count (case insensitive)');
    $Search->delete;


    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'ObjectID' => $Object->id,
        'SearchDefinition' => GetSearchDefinition(LoadClassByAlias('Object'), 'LocalizedText'),
        'Parameters' => {
            'String'   => '%all%',
            'LanguageID' => GetPKeyLanguageByCode('en'),
            'Class.Alias' => 'TestObject',
        },
        'SortParameters'=> [
            'Class.Alias',
            'Alias'
        ],
    });
    $Search->statistic;
    ok ($Search->countItems == 0, 'found correct count');
    $Search->delete;

    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'ObjectID' => $Object->id,
        'SearchDefinition' => GetSearchDefinition(LoadClassByAlias('Object'), 'LocalizedText'),
        'Parameters' => {
            'String'   => '%TagList%',
            'LanguageID' => GetPKeyLanguageByCode('de'),
            'Class.Alias' => 'TestObject',
        },
        'SortParameters'=> [
            'Class.Alias',
            'Alias'
        ],
    });
    $Search->statistic;
    ok ($Search->countItems == 6, 'found correct count');
    $Search->delete;

    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'ObjectID' => $Object->id,
        'SearchDefinition' => GetSearchDefinition(LoadClassByAlias('Object'), 'LocalizedText'),
        'Parameters' => {
            'String'   => '%XXXX%',
            'LanguageID' => GetPKeyLanguageByCode('de'),
            'Class.Alias' => 'TestObject',
        },
        'SortParameters'=> [
            'Class.Alias',
            'Alias'
        ],
    });
    $Search->statistic;
    ok ($Search->countItems == 0, 'found correct count');
    $Search->delete;

    ok(Cleanup(), "Cleanup");

    DisconnectSite();
});

sub Cleanup {
    DeleteObjects('import.xml', $ROOTPATH );
    return 1;
}

sub Init {
    Cleanup();
    ImportObjects('import.xml', $ROOTPATH );
    return 1;
}

1;
