use strict;
use Test::More tests => 11;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadRootObject );


sub Main {

    my $TestShop = LoadObjectByPath("/Test/TestShop1");
    ok( $TestShop->id, "create shop" );
    is( $TestShop->featureCurrentValue('Regions'), 0, "initial feature count" );

    # add, test and remove regions
    $TestShop->addRegion('Inland');
    ok($TestShop->existsRegion('Inland'),"addRegion");
    ok(! $TestShop->existsRegion('Ausland'),"not exists Region");
    is( $TestShop->featureCurrentValue('Regions'), 1, "featureCurrentValue" );
    is( scalar @{$TestShop->regions}, 1, "scalar regions" );
    is( $TestShop->regions->[0]->alias, 'Inland', "regions" );
    $TestShop->removeRegion('Inland');
    ok(! $TestShop->existsRegion('Inland'), "removeRegion");

    # add countries to shop
    $TestShop->addCountry(826, 10, 0) unless $TestShop->existsCountry(826);

    # assign countries to regions
    my $Region = $TestShop->addRegion('Ausland');
    $Region->addCountry(826);
    ok($Region->existsCountry(826),"add country");

    my $Region1 = $TestShop->addRegion('Welt');
    $Region1->addCountry(826);
    ok($Region1->existsCountry(826),"add country");
    $Region1->removeCountry(826);
    ok(! $Region1->existsCountry(826), "remove country");
}

sub Init {

    Cleanup();

    my $System = LoadRootObject();
    my $TestShop = $System->folder('Test')->insertChild( 'Shop', {
        Alias => 'TestShop1'
    });

    my $Feature = LoadObjectByPath('/Features/Regions');
    $TestShop->featureMaxValue( $Feature, 5 );
}

sub Cleanup {

    my $System = LoadRootObject();
    $System->child( 'Test' )->delete if $System->existsChild( 'Test' );
}

RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        Init();
        Main();
        Cleanup();
    });
});
