use Test::Simple tests => 14;
use strict;
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Object::API::Factory  qw (Transaction);
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use Getopt::Long;
use DE_EPAGES::Shop::API::ShopChildOrderHandler qw (
    RegisterAliasInShopChildOrder
    GetShopChildOrderList
    PrintShopChildOrderListToFile
);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (GetInfoCartridgeByPackage);
use DE_EPAGES::Shop::API::Table::ShopChildOrder qw (
    UpdateShopChildOrder
    DeleteShopChildOrder
    GetInfoShopChildOrder
    ExistsShopChildOrder
    GetInfoShopChildOrderByShopChildAlias
    GetInfoShopChildOrderBySuccessorID
);

use DE_EPAGES::Shop::API::Table::ShopChildDependency qw(
    ExistsShopChildDependency
);
use DE_EPAGES::Shop::API::XML::ShopChildOrderImportHandler qw (
    ImportShopChildDependency
);
use DE_EPAGES::Shop::API::XML::ShopChildOrderDeleteHandler qw (
    DeleteShopChildDependencies
);
use DE_EPAGES::Core::API::File qw ( GetFileContent DeleteFile );

local $ENV{'EPAGES_TESTSITE'} = 'Store';

sub Main {
  my $StoreName = 'Store';
  my $ProviderName = 'Distributor';
  my $Password;
  my $Help;
  my $OutputFileName = 'D:/Test2.txt';

  GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'outputfile=s'  => \$OutputFileName,
        'storename'     => \$StoreName
    );
    usage() if $Help;

    my $time1 = time;

    #get original state of list
    PrintShopChildOrderListToFile('TargetStateOfShopChildOrder.txt');
    my $sTargetStateOfShopChildOrder = GetFileContent('TargetStateOfShopChildOrder.txt');

    my $aUnusedOrderList = GetShopChildOrderList();


    #TEST-01:insert in list by alphabetic order
    CleanOrderList();
    my @Alphabet = ("A".."Z");
    my $ShopCartrID = GetInfoCartridgeByPackage('DE_EPAGES::Shop')->{'CartridgeID'};
    my @Array;

    foreach my $Letter (reverse @Alphabet){
        RegisterAliasInShopChildOrder($Letter . '@_Test',[], $ShopCartrID);
    }
    foreach my $Letter (@Alphabet){
        push(@Array, $Letter . '@_Test');
    }

    ok(CompareArrays(\@Array,GetTestOrder()), "Alphabetic sort without dependencies");

    #TEST-02: insert with dependencies
    CleanOrderList();

    RegisterAliasInShopChildOrder('A@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('B@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('C@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('AA@_Test',['A@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('E@_Test',['A@_Test','C@_Test'], $ShopCartrID);

    @Array = ('A@_Test','AA@_Test','B@_Test','C@_Test','E@_Test');

    ok(CompareArrays(\@Array, GetTestOrder() ), "correct insert with dependencies");

    #TEST-03: insert with dependencies
    CleanOrderList();

    RegisterAliasInShopChildOrder('Z@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('Y@_Test',['Z@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('X@_Test',['Y@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('W@_Test',['X@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('V@_Test',['W@_Test'], $ShopCartrID);

    @Array = ('Z@_Test','Y@_Test','X@_Test','W@_Test','V@_Test');

    ok(CompareArrays(\@Array, GetTestOrder() ), "correct insert with dependencies part 2");

    #TEST-04:insert by enlarge dependencies
    CleanOrderList();

    RegisterAliasInShopChildOrder('A@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('B@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('C@_Test',['A@_Test','B@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('D@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('E@_Test',['C@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('F@_Test',['C@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('G@_Test',['E@_Test'], $ShopCartrID);

    #reinsert with more dependencies
    RegisterAliasInShopChildOrder('C@_Test',['A@_Test','B@_Test','D@_Test'], $ShopCartrID);

    @Array = ('A@_Test','B@_Test','D@_Test','C@_Test','E@_Test','F@_Test','G@_Test');

    ok(CompareArrays(\@Array, GetTestOrder() ), "correct insert C enlarge dependencies");

    #TEST-05: insert with smaller dependencies
    CleanOrderList();

    RegisterAliasInShopChildOrder('A@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('B@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('C@_Test',['A@_Test','B@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('D@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('E@_Test',['C@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('F@_Test',['C@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('G@_Test',['E@_Test'], $ShopCartrID);

    RegisterAliasInShopChildOrder('C@_Test',['A@_Test'], $ShopCartrID);

    @Array = ('A@_Test','B@_Test','C@_Test','D@_Test','E@_Test','F@_Test','G@_Test');

    ok(CompareArrays(\@Array, GetTestOrder() ), "correct insert C with smaller dependencies");

    #TEST-06: important test that move all successor nodes that where infront the last constraint node, behind the last constraint node
    #otherwise adding A@_Test with enlarged dependencies will failed
    CleanOrderList();
    RegisterAliasInShopChildOrder('A@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('B@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('C@_Test',['A@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('D@_Test',['B@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('E@_Test',['C@_Test'], $ShopCartrID);

    RegisterAliasInShopChildOrder('A@_Test',['B@_Test', 'D@_Test'], $ShopCartrID);

    @Array = ('B@_Test','D@_Test','A@_Test','C@_Test','E@_Test');

    ok(CompareArrays(\@Array, GetTestOrder() ), "correct insert move direct/indirekt successor behind all superior nodes");

    #TEST-07: same as TEST-06
    CleanOrderList();
    RegisterAliasInShopChildOrder('A@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('B@_Test',['A@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('C@_Test',['B@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('D@_Test',['C@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('X@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('Y@_Test',['X@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('Z@_Test',['Y@_Test'], $ShopCartrID);

    RegisterAliasInShopChildOrder('A@_Test',['X@_Test', 'Y@_Test', 'Z@_Test'], $ShopCartrID);

    @Array = ('X@_Test','Y@_Test','Z@_Test','A@_Test','B@_Test', 'C@_Test', 'D@_Test');

    ok(CompareArrays(\@Array, GetTestOrder() ), "correct insert move direct/indirekt successor behind all superior nodes");

    #TEST-08: Test if circle is not accepted
    CleanOrderList();
    RegisterAliasInShopChildOrder('A@_Test',[], $ShopCartrID);
    RegisterAliasInShopChildOrder('B@_Test',['A@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('C@_Test',['B@_Test'], $ShopCartrID);
    RegisterAliasInShopChildOrder('D@_Test',['C@_Test'], $ShopCartrID);

    eval{
        RegisterAliasInShopChildOrder('A@_Test',['D@_Test'], $ShopCartrID);
    };
    ok(ExistsError(), "insert not accepted if insert creates a circle");

    #TEST-09: praxis Test import XML file
    CleanOrderList();
    ImportShopChildDependency('ShopChildOrder-TestImport.xml');

    @Array = (
          'Address@_Test',
          'AnonymousUsers@_Test',
          'BusinessHours@_Test',
          'CouponCampaigns@_Test',
          'CustomerClass@_Test',
          'CustomerGroups@_Test',
          'EbayDefaultAuction@_Test',
          'EbayTemplates@_Test',
          'EbayUserTemplates@_Test',
          'InterestRateMatrices@_Test',
          'Portals@_Test',
          'ProductClass@_Test',
          'ProductTypes@_Test',
          'Appointments@_Test',
          'ShippingMethods@_Test',
          'MailTypes@_Test',
          'MediaGallery@_Test',
          'NewsletterCampaigns@_Test',
          'Regions@_Test',
          'PaymentMethods@_Test',
          'ShippingOptions@_Test',
          'Users@_Test',
          'Groups@_Test',
          'NewsletterCampaign@_Test',
          'PriceLists@_Test',
          'Products@_Test',
          'Categories@_Test',
          'EbayAuctions@_Test',
          'Customers@_Test',
          'PortalConfigs@_Test',
          'Styles@_Test'
    );
    ok(CompareArrays(\@Array, GetTestOrder() ), "correct insert from xml file");

    #TEST- : Dependency exists
    my $ID1 = GetInfoShopChildOrderByShopChildAlias('EbayAuctions@_Test')->{'ShopChildID'};
    my $ID2 = GetInfoShopChildOrderByShopChildAlias('Customers@_Test')->{'ShopChildID'};
    my $TempCartridgeID = GetInfoCartridgeByPackage('DE_EPAGES::Ebay')->{'CartridgeID'};

    ok(ExistsShopChildDependency($ID1,$ID2,$TempCartridgeID), "dependency exists");

    #TEST-10: praxis Test delete from XML file
    DeleteShopChildDependencies('ShopChildOrder-TestDelete.xml');

    @Array = (
          'Address@_Test',
          'AnonymousUsers@_Test',
          'BusinessHours@_Test',
          'CouponCampaigns@_Test',
          'CustomerClass@_Test',
          'CustomerGroups@_Test',
          'InterestRateMatrices@_Test',
          'Portals@_Test',
          'ProductClass@_Test',
          'ProductTypes@_Test',
          'Appointments@_Test',
          'ShippingMethods@_Test',
          'MailTypes@_Test',
          'MediaGallery@_Test',
          'NewsletterCampaigns@_Test',
          'Regions@_Test',
          'PaymentMethods@_Test',
          'ShippingOptions@_Test',
          'Users@_Test',
          'Groups@_Test',
          'NewsletterCampaign@_Test',
          'PriceLists@_Test',
          'Products@_Test',
          'Categories@_Test',
          'Customers@_Test',
          'PortalConfigs@_Test',
          'Styles@_Test'
    );

    ok(CompareArrays(\@Array, GetTestOrder() ), "correct delete from xml file");


    #Dependency Test
    ok(! ExistsShopChildDependency($ID1,$ID2,$TempCartridgeID), "dependency correct deleted");

    CleanOrderList();

    #TEST-10: check if test doesnt affect orderlist
    ok(CompareArrays($aUnusedOrderList,GetShopChildOrderList()), "test had no effect on shopchildorderlist 1st");

    #get actual state of list
    PrintShopChildOrderListToFile('ActualStateOfShopChildOrder.txt');
    my $sActualStateOfShopChildOrder = GetFileContent('ActualStateOfShopChildOrder.txt');

    #look for no effects on shop child order list
    my $IsEqual = $$sActualStateOfShopChildOrder eq $$sTargetStateOfShopChildOrder;
    ok($IsEqual, "test had no effect on shopchildorderlist 2nd");

    DeleteFile('ActualStateOfShopChildOrder.txt') if $IsEqual;
    DeleteFile('TargetStateOfShopChildOrder.txt') if $IsEqual;

    my $time2 = time;
    my $seconds = $time2-$time1;
    my $logmsg = sprintf ("need %.3f seconds", $seconds);
    print $logmsg."\n";

    Warning( GetError() ) if ExistsError();
}

sub GetTestOrder{
    my @ResultArray;
    foreach my $Alias(@{GetShopChildOrderList()}){
        push(@ResultArray, $Alias) if ($Alias =~ /\@\_Test$/);
    }
    return \@ResultArray;
}

sub CompareArrays{
    my ($aArray1,$aArray2) = @_;
    return 0 if (scalar @{$aArray1} != scalar @{$aArray2});
    for(my $i=0; $i < scalar @{$aArray1}; $i++){
        return 0 if (@{$aArray1}[$i] ne @{$aArray2}[$i]);
    }
    return 1;
}

sub CleanOrderList{
    foreach my $Alias (@{GetShopChildOrderList()}){
        if ($Alias =~ /\@\_Test$/){
            my $hDeleteInfo = GetInfoShopChildOrderByShopChildAlias($Alias);
            CutNode($hDeleteInfo->{'ShopChildID'});
            DeleteShopChildOrder($hDeleteInfo->{'ShopChildID'}) if (ExistsShopChildOrder($hDeleteInfo->{'ShopChildID'}));
        }
    }
}

sub CutNode{
    my ($NodeID) = @_;
    my $Node = GetInfoShopChildOrder($NodeID);

    my $ParentNodeID = GetInfoShopChildOrderBySuccessorID($NodeID)->{'ShopChildID'} or die "no ParentID where found";
    my $SuccessorNodeID = $Node->{'SuccessorID'};

    Transaction( sub {
        #node successor temporary reference to startnode, because successorid is unique
        UpdateShopChildOrder($NodeID,{'SuccessorID' => GetInfoShopChildOrderByShopChildAlias('STARTNODE')->{'ShopChildID'}});
        #take node away from order
        UpdateShopChildOrder($ParentNodeID,{'SuccessorID' => $SuccessorNodeID});}
    );

}


sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -outputfile   outputfile path                     (D:/Test.txt)
    -storename    name of database                    (Store)
flags:
    -help         show the command line options

Example:
    perl $0   -outputfile D:/Test.txt
END_USAGE
    exit 2;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});