use strict;
use Test::Simple tests => 6;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject);

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $System = LoadRootObject();
    my $Shop = $System->child( 'TestShop' );
    ok( $Shop->id, "create shop" );

    $Shop->featureMaxValue('Newsletter', 1);

    my $CampaignFolder = $Shop->folder('NewsletterCampaigns');
    my $Campaign = $CampaignFolder->insertChild('NewsletterCampaign', {'Alias' => 'TestCampaign', 'Name' => 'TestCampaign'});
    ok( $Campaign->id, "create campaign" );

    my $NewsletterFolder = $Campaign->folder('Newsletters');
    my $Newsletter = $NewsletterFolder->insertChild('Newsletter', {'Alias' => 'TestNewsletter', 'Name' => 'TestNewsletter'});
    ok( $Newsletter->id, "create newsletter" );

    my $AddresseeFolder = $Campaign->folder('Addressees');
    my $Addressee = $AddresseeFolder->insertChild('NewsletterAddressee', {'Alias' => 'TestAddressee', 'Name' => 'TestAddressee', 'EMail' => 'TestEMail'});
    ok( $Addressee->id, "create addressee" );

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    my $System = LoadRootObject();
    if( $System->existsChild( 'TestShop' ) ) {
        $System->child( 'TestShop' )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Shop', { Alias => 'TestShop' } );

    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
