use Test;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectToSite DisconnectSite );
use DE_EPAGES::Trigger::API::Table::Hook qw ( DeleteHook InsertHook GetPKeyHooksByName );
use DE_EPAGES::Trigger::API::Table::HookFunction qw ( InsertHookFunction );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

our $CALLED = 0;

sub Main {
    plan tests => 1;

    local $| = 1;
    local $ENV{SCRIPT_NAME} = __FILE__;
    ConnectToSite();
    Cleanup();
    ok ( Test_Package1() );
    Cleanup();
    DisconnectSite();
}

sub Test_Package1 {
    eval {
        my $HookID = InsertHook({'Name'=>'TestHook'});
        Error('ID_NOT_INT') unless $HookID =~ /^\d+$/;
        InsertHookFunction({
            'HookID' => $HookID,
            'OrderNo' => 1,
            'FunctionName' => 'TestHookProc',
        });
        my $HookID2 = InsertHook({'Name'=>'TestHookPackage'});
        Error('ID_NOT_INT') unless $HookID2 =~ /^\d+$/;
        InsertHookFunction({
            'HookID' => $HookID2,
            'OrderNo' => 1,
            'FunctionName' => 'DE_EPAGES::Trigger::t::HookT::TestHookProc',
        });
        TriggerHook('TestHook', { 'TestParameter'=>'Param' });
        Error('NO_HOOK_NOTCALLED') unless $CALLED;
        $DE_EPAGES::Trigger::t::HookT::CALLED = 0;
        TriggerHook('TestHookPackage', { 'TestParameter'=>'Param' });
        Error('NO_HOOK_NOTCALLED') unless $DE_EPAGES::Trigger::t::HookT::CALLED;
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub TestHookProc {
    my ($args) = @_;
    Error('NO_HOOK_NAME') unless defined $args->{'HookName'}      and $args->{'HookName'} eq 'TestHook';
    Error('NO_HOOK_PARA') unless defined $args->{'TestParameter'} and $args->{'TestParameter'} eq 'Param';
    $CALLED = 1;
}

sub Cleanup {
    my $IDs = GetPKeyHooksByName('TestHook');
    push @$IDs, @{GetPKeyHooksByName('TestHookPackage')};
    DeleteHook($_) foreach @$IDs;
}

RunScript(
    'Sub' => \&Main
);

1;

