use strict;
use Test::More tests => 4;

use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::TrustedRating::API::TrustedRating qw ( UpdateRatingState );

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store';

sub Main {
    ConnectTestSite();

    eval {UpdateRatingState('X41495A6E65ECDDCD554A02C0601D1C97', 0) };
    ok (!ExistsError(), 'TS-ID is ok. State has changed to inactive.');
    eval {UpdateRatingState('X41495A6E65ECDDCD554A02C0601D1C97', 1) };
    ok (!ExistsError(), 'TS-ID is ok. State has changed to active.');
    eval {UpdateRatingState('X41495A6E65ECDDCD554A02C0601D', 0)};
    ok (ExistsError && GetError()->code eq 'INVALID_TSID', 'TS-ID is invalid.');
    eval {UpdateRatingState('', 0)};
    ok (ExistsError && GetError()->code eq 'STRING_EMPTY', 'TS-ID is empty.');

    DisconnectSite();
}

RunScript(Sub=>\&Main, Debug=>1);