use strict;
use Test::Simple tests => 3;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use Data::Dumper;

use SOAP::Lite
  on_fault => sub {
    my $soap = shift;
    my $res = shift;
    ref $res ? die "SOAP FAULT: " . $res->faultstring . "\n"
             : die "TRANSPORT ERROR: " . $soap->transport->status . "\n";
    return $res;
  },
;

sub Main {
    my $StoreName = GetTestSite();
    my $soap = SOAP::Lite
        ->uri('urn://epages.de/WebService/AsyncExampleService/2010/06')
        ->proxy("http://localhost/epages/$StoreName.soap");

    my $Count = 3;

    my $sessionID = $soap->startLongTimeFunc($Count)->result;
    ok( $sessionID, 'long running function startet' );

    my $status = 1;
    while ($status<$Count) {
        $status = $soap->getSessionStatus($sessionID)->result->{'Status'};
        sleep(5);
    }
    ok( $status >= $Count, 'session status changed' );

    my $deleted = $soap->deleteSession($sessionID)->result;
    ok( $deleted eq 'deleted', 'session deleted' );

}

sub Initial {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        ImportObjects('../Samples/Server/ActionsWebServices.xml');
    });
}

sub Cleanup {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        DeleteObjects('../Samples/Server/ActionsWebServices.xml');
    });
}

RunScript(Sub => sub {
    Cleanup();
    Initial();
    Main();
    Cleanup();
}, Debug=>1);
