use strict;
use Test::More tests => 8;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadRootObject LoadObjectByPath );

my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    ok( Init(), "initialization" );

    my $WSessionFolder = LoadRootObject()->folder('WebServiceSessions');
    my $WSession = $WSessionFolder->insertChild('WebServiceSession');
    ok ( $WSession, 'web service session created' );

    my $WAlias = $WSession->alias;
    ok ( $WAlias, 'web service sessionid=alias='.$WAlias );
    ok ( $WSessionFolder->existsChild($WAlias), 'session is child of web service session folder' );

    $WSession->set({'Status'=>2});
    is ( $WSession->get('Status'), 2, 'web service session status changed to 2' );
    $WSession->set({'StatusMessage'=>'OK'});
    ok ( $WSession->get('StatusMessage') eq 'OK', 'web service session status message changed' );

    $WSession->delete;
    ok ( !$WSessionFolder->existsChild($WAlias), 'session is NOT child of web service session folder' );

    ok( Cleanup(), "cleanup" );

    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();

    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
